from typing import List, Callable
from easy_vk.bot import update_types
import re


class UpdateHandlerBase:
    def __init__(self, update_type: str, update_object, filters: List[Callable], function: Callable):
        self.update_type = update_type
        self.update_object = update_object
        self.filters = filters
        self.function = function

    def check(self, update):
        if update['type'] != self.update_type:
            return False
        update = self.update_object(**update['object'])
        for f in self.filters:
            if not f(update):
                return False
        return True

    def handle(self, update):
        update = self.update_object(**update['object'])
        self.function(update)


class Handlers:
    def __init__(self):
        self._handler_list: List[UpdateHandlerBase] = []

    def message_new(self,
                    regexp: str = None,
                    user_id: int = None,
                    user_ids: List[int] = None,
                    user_black_list: List[int] = None):
        """
        Handles new message

        :param regexp: filter of message text
        :param user_id: filter of writer user id
        :param user_ids: filter of writers user ids
        :param user_black_list: filter of blacklisted writers
        """

        if user_id:
            user_ids = [user_id]

        def decorator(function):
            filters = []

            # adding filters
            if regexp:
                filters.append(lambda update: re.fullmatch(regexp, update.message.text, flags=re.MULTILINE))
            if user_ids:
                filters.append(lambda update: update.message.from_id in user_ids)
            if user_black_list:
                filters.append(lambda update: not update.message.from_id in user_black_list)

            self._handler_list.append(UpdateHandlerBase('message_new', update_types.MessageNew, filters, function))
            return function

        return decorator

    def handle(self, update):
        for handler in self._handler_list:
            if handler.check(update):
                handler.handle(update)



# This file was autogenerated from vk-api json schema

from typing import List, Union, Optional, overload
from easy_vk.types import objects
from easy_vk.types import responses
from easy_vk.api_category import BaseCategory
try:
    from typing import Literal
except Exception:
    from typing_extensions import Literal


class Messages(BaseCategory):
    def __init__(self, session, access_token: str, v: str, last_call_timer, delay: float, auto_retry: bool, max_retries: int, timeout: float):
        super().__init__(session, access_token, v, last_call_timer, delay, auto_retry, max_retries, timeout)

    def create_chat(self, user_ids: Optional[List[int]] = None, title: Optional[str] = None, group_id: Optional[int] = None) -> responses.MessagesCreateChat:
        """
        Creates a chat with several participants.
        
        :param user_ids: IDs of the users to be added to the chat.
        :param title: Chat title.
        :param group_id:
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'messages.createChat'
        response_type = responses.MessagesCreateChat
        return self._call(method_name, method_parameters, param_aliases, response_type)

    # def delete(self, message_ids: Optional[List[int]] = None, spam: Optional[bool] = None, group_id: Optional[int] = None, delete_for_all: Optional[bool] = None) -> responses.MessagesDelete:
    #     """
    #     Deletes one or more messages.
    #
    #     :param message_ids: Message IDs.
    #     :param spam: '1' — to mark message as spam.
    #     :param group_id: Group ID (for group messages with user access token)
    #     :param delete_for_all: '1' — delete message for for all.
    #     """
    #
    #     method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
    #     param_aliases = []
    #     method_name = 'messages.delete'
    #     response_type = responses.MessagesDelete
    #     return self._call(method_name, method_parameters, param_aliases, response_type)

    def delete_chat_photo(self, chat_id: int, group_id: Optional[int] = None) -> responses.MessagesDeleteChatPhoto:
        """
        Deletes a chat's cover picture.
        
        :param chat_id: Chat ID.
        :param group_id:
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'messages.deleteChatPhoto'
        response_type = responses.MessagesDeleteChatPhoto
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def delete_conversation(self, user_id: Optional[int] = None, peer_id: Optional[int] = None, group_id: Optional[int] = None) -> responses.MessagesDeleteConversation:
        """
        Deletes all private messages in a conversation.
        
        :param user_id: User ID. To clear a chat history use 'chat_id'
        :param peer_id: Destination ID. "For user: 'User ID', e.g. '12345'. For chat: '2000000000' + 'chat_id', e.g. '2000000001'. For community: '- community ID', e.g. '-12345'. "
        :param group_id: Group ID (for group messages with user access token)
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'messages.deleteConversation'
        response_type = responses.MessagesDeleteConversation
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def edit(self, peer_id: int, message: Optional[str] = None, lat: Optional[float] = None, long: Optional[float] = None, attachment: Optional[str] = None, keep_forward_messages: Optional[bool] = None, keep_snippets: Optional[bool] = None, group_id: Optional[int] = None, dont_parse_links: Optional[bool] = None, message_id: Optional[int] = None, conversation_message_id: Optional[int] = None, template: Optional[str] = None, keyboard: Optional[str] = None) -> responses.MessagesEdit:
        """
        Edits the message.
        
        :param peer_id: Destination ID. "For user: 'User ID', e.g. '12345'. For chat: '2000000000' + 'chat_id', e.g. '2000000001'. For community: '- community ID', e.g. '-12345'. "
        :param message: (Required if 'attachments' is not set.) Text of the message.
        :param lat: Geographical latitude of a check-in, in degrees (from -90 to 90).
        :param long: Geographical longitude of a check-in, in degrees (from -180 to 180).
        :param attachment: (Required if 'message' is not set.) List of objects attached to the message, separated by commas, in the following format: "<owner_id>_<media_id>", '' — Type of media attachment: 'photo' — photo, 'video' — video, 'audio' — audio, 'doc' — document, 'wall' — wall post, '<owner_id>' — ID of the media attachment owner. '<media_id>' — media attachment ID. Example: "photo100172_166443618"
        :param keep_forward_messages: '1' — to keep forwarded, messages.
        :param keep_snippets: '1' — to keep attached snippets.
        :param group_id: Group ID (for group messages with user access token)
        :param dont_parse_links:
        :param message_id:
        :param conversation_message_id:
        :param template:
        :param keyboard:
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'messages.edit'
        response_type = responses.MessagesEdit
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def edit_chat(self, chat_id: int, title: Optional[str] = None) -> responses.BaseOk:
        """
        Edits the title of a chat.
        
        :param chat_id: Chat ID.
        :param title: New title of the chat.
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'messages.editChat'
        response_type = responses.BaseOk
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def get_by_conversation_message_id(self, peer_id: int, conversation_message_ids: List[int], extended: Optional[bool] = None, fields: Optional[List[Union[objects.UsersFields, str]]] = None, group_id: Optional[int] = None) -> responses.MessagesGetByConversationMessageId:
        """
        Returns messages by their IDs within the conversation.
        
        :param peer_id: Destination ID. "For user: 'User ID', e.g. '12345'. For chat: '2000000000' + 'chat_id', e.g. '2000000001'. For community: '- community ID', e.g. '-12345'. "
        :param conversation_message_ids: Conversation message IDs.
        :param extended: Information whether the response should be extended
        :param fields: Profile fields to return.
        :param group_id: Group ID (for group messages with group access token)
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'messages.getByConversationMessageId'
        response_type = responses.MessagesGetByConversationMessageId
        return self._call(method_name, method_parameters, param_aliases, response_type)

    @overload
    def get_by_id(self, message_ids: List[int], preview_length: Optional[int] = None, extended: None = None, fields: Optional[List[Union[objects.UsersFields, str]]] = None, group_id: Optional[int] = None) -> responses.MessagesGetById: ...
    @overload
    def get_by_id(self, message_ids: List[int], extended: bool, preview_length: Optional[int] = None, fields: Optional[List[Union[objects.UsersFields, str]]] = None, group_id: Optional[int] = None) -> responses.MessagesGetByIdExtended: ...
    def get_by_id(self, message_ids: List[int], preview_length: Optional[int] = None, extended: Optional[bool] = None, fields: Optional[List[Union[objects.UsersFields, str]]] = None, group_id: Optional[int] = None):
        """
        Returns messages by their IDs.
        
        :param message_ids: Message IDs.
        :param preview_length: Number of characters after which to truncate a previewed message. To preview the full message, specify '0'. "NOTE: Messages are not truncated by default. Messages are truncated by words."
        :param extended: Information whether the response should be extended
        :param fields: Profile fields to return.
        :param group_id: Group ID (for group messages with group access token)
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'messages.getById'
        if not extended:
            response_type = responses.MessagesGetById
        if extended:
            response_type = responses.MessagesGetByIdExtended
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def get_conversation_members(self, peer_id: int, fields: Optional[List[Union[objects.UsersFields, str]]] = None, group_id: Optional[int] = None) -> responses.MessagesGetConversationMembers:
        """
        Returns a list of IDs of users participating in a chat.
        
        :param peer_id: Peer ID.
        :param fields: Profile fields to return.
        :param group_id: Group ID (for group messages with group access token)
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'messages.getConversationMembers'
        response_type = responses.MessagesGetConversationMembers
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def get_conversations(self, offset: Optional[int] = None, count: Optional[int] = None, filter_: Optional[str] = None, extended: Optional[bool] = None, start_message_id: Optional[int] = None, fields: Optional[List[Union[objects.BaseUserGroupFields, str]]] = None, group_id: Optional[int] = None) -> responses.MessagesGetConversations:
        """
        Returns a list of the current user's conversations.
        
        :param offset: Offset needed to return a specific subset of conversations.
        :param count: Number of conversations to return.
        :param filter_: Filter to apply: 'all' — all conversations, 'unread' — conversations with unread messages, 'important' — conversations, marked as important (only for community messages), 'unanswered' — conversations, marked as unanswered (only for community messages)
        :param extended: '1' — return extra information about users and communities
        :param start_message_id: ID of the message from what to return dialogs.
        :param fields: Profile and communities fields to return.
        :param group_id: Group ID (for group messages with group access token)
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = [('filter_', 'filter')]
        method_name = 'messages.getConversations'
        response_type = responses.MessagesGetConversations
        return self._call(method_name, method_parameters, param_aliases, response_type)

    @overload
    def get_conversations_by_id(self, peer_ids: List[int], extended: None = None, fields: Optional[List[Union[objects.BaseUserGroupFields, str]]] = None, group_id: Optional[int] = None) -> responses.MessagesGetConversationsById: ...
    @overload
    def get_conversations_by_id(self, peer_ids: List[int], extended: bool, fields: Optional[List[Union[objects.BaseUserGroupFields, str]]] = None, group_id: Optional[int] = None) -> responses.MessagesGetConversationsByIdExtended: ...
    def get_conversations_by_id(self, peer_ids: List[int], extended: Optional[bool] = None, fields: Optional[List[Union[objects.BaseUserGroupFields, str]]] = None, group_id: Optional[int] = None):
        """
        Returns conversations by their IDs
        
        :param peer_ids: Destination IDs. "For user: 'User ID', e.g. '12345'. For chat: '2000000000' + 'chat_id', e.g. '2000000001'. For community: '- community ID', e.g. '-12345'. "
        :param extended: Return extended properties
        :param fields: Profile and communities fields to return.
        :param group_id: Group ID (for group messages with group access token)
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'messages.getConversationsById'
        if not extended:
            response_type = responses.MessagesGetConversationsById
        if extended:
            response_type = responses.MessagesGetConversationsByIdExtended
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def get_history(self, offset: Optional[int] = None, count: Optional[int] = None, user_id: Optional[int] = None, peer_id: Optional[int] = None, start_message_id: Optional[int] = None, rev: Optional[int] = None, extended: Optional[bool] = None, fields: Optional[List[Union[objects.UsersFields, str]]] = None, group_id: Optional[int] = None) -> responses.MessagesGetHistory:
        """
        Returns message history for the specified user or group chat.
        
        :param offset: Offset needed to return a specific subset of messages.
        :param count: Number of messages to return.
        :param user_id: ID of the user whose message history you want to return.
        :param peer_id:
        :param start_message_id: Starting message ID from which to return history.
        :param rev: Sort order: '1' — return messages in chronological order. '0' — return messages in reverse chronological order.
        :param extended: Information whether the response should be extended
        :param fields: Profile fields to return.
        :param group_id: Group ID (for group messages with group access token)
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'messages.getHistory'
        response_type = responses.MessagesGetHistory
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def get_history_attachments(self, peer_id: int, media_type: Optional[str] = None, start_from: Optional[str] = None, count: Optional[int] = None, photo_sizes: Optional[bool] = None, fields: Optional[List[Union[objects.UsersFields, str]]] = None, group_id: Optional[int] = None, preserve_order: Optional[bool] = None, max_forwards_level: Optional[int] = None) -> responses.MessagesGetHistoryAttachments:
        """
        Returns media files from the dialog or group chat.
        
        :param peer_id: Peer ID. ", For group chat: '2000000000 + chat ID' , , For community: '-community ID'"
        :param media_type: Type of media files to return: *'photo',, *'video',, *'audio',, *'doc',, *'link'.,*'market'.,*'wall'.,*'share'
        :param start_from: Message ID to start return results from.
        :param count: Number of objects to return.
        :param photo_sizes: '1' — to return photo sizes in a
        :param fields: Additional profile [vk.com/dev/fields|fields] to return. 
        :param group_id: Group ID (for group messages with group access token)
        :param preserve_order:
        :param max_forwards_level:
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'messages.getHistoryAttachments'
        response_type = responses.MessagesGetHistoryAttachments
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def get_invite_link(self, peer_id: int, reset: Optional[bool] = None, group_id: Optional[int] = None) -> responses.MessagesGetInviteLink:
        """
        :param peer_id: Destination ID.
        :param reset: 1 — to generate new link (revoke previous), 0 — to return previous link.
        :param group_id: Group ID
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'messages.getInviteLink'
        response_type = responses.MessagesGetInviteLink
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def get_long_poll_history(self, ts: Optional[int] = None, pts: Optional[int] = None, preview_length: Optional[int] = None, onlines: Optional[bool] = None, fields: Optional[List[Union[objects.UsersFields, str]]] = None, events_limit: Optional[int] = None, msgs_limit: Optional[int] = None, max_msg_id: Optional[int] = None, group_id: Optional[int] = None, lp_version: Optional[int] = None, last_n: Optional[int] = None, credentials: Optional[bool] = None) -> responses.MessagesGetLongPollHistory:
        """
        Returns updates in user's private messages.
        
        :param ts: Last value of the 'ts' parameter returned from the Long Poll server or by using [vk.com/dev/messages.getLongPollHistory|messages.getLongPollHistory] method.
        :param pts: Lsat value of 'pts' parameter returned from the Long Poll server or by using [vk.com/dev/messages.getLongPollHistory|messages.getLongPollHistory] method.
        :param preview_length: Number of characters after which to truncate a previewed message. To preview the full message, specify '0'. "NOTE: Messages are not truncated by default. Messages are truncated by words."
        :param onlines: '1' — to return history with online users only.
        :param fields: Additional profile [vk.com/dev/fields|fields] to return.
        :param events_limit: Maximum number of events to return.
        :param msgs_limit: Maximum number of messages to return.
        :param max_msg_id: Maximum ID of the message among existing ones in the local copy. Both messages received with API methods (for example, , ), and data received from a Long Poll server (events with code 4) are taken into account.
        :param group_id: Group ID (for group messages with user access token)
        :param lp_version:
        :param last_n:
        :param credentials:
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'messages.getLongPollHistory'
        response_type = responses.MessagesGetLongPollHistory
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def get_long_poll_server(self, need_pts: Optional[bool] = None, group_id: Optional[int] = None, lp_version: Optional[int] = None) -> responses.MessagesGetLongPollServer:
        """
        Returns data required for connection to a Long Poll server.
        
        :param need_pts: '1' — to return the 'pts' field, needed for the [vk.com/dev/messages.getLongPollHistory|messages.getLongPollHistory] method.
        :param group_id: Group ID (for group messages with user access token)
        :param lp_version: Long poll version
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'messages.getLongPollServer'
        response_type = responses.MessagesGetLongPollServer
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def is_messages_from_group_allowed(self, group_id: int, user_id: int) -> responses.MessagesIsMessagesFromGroupAllowed:
        """
        Returns information whether sending messages from the community to current user is allowed.
        
        :param group_id: Group ID.
        :param user_id: User ID.
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'messages.isMessagesFromGroupAllowed'
        response_type = responses.MessagesIsMessagesFromGroupAllowed
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def mark_as_answered_conversation(self, peer_id: int, answered: Optional[bool] = None, group_id: Optional[int] = None) -> responses.BaseOk:
        """
        Marks and unmarks conversations as unanswered.
        
        :param peer_id: ID of conversation to mark as important.
        :param answered: '1' — to mark as answered, '0' — to remove the mark
        :param group_id: Group ID (for group messages with group access token)
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'messages.markAsAnsweredConversation'
        response_type = responses.BaseOk
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def mark_as_important_conversation(self, peer_id: int, important: Optional[bool] = None, group_id: Optional[int] = None) -> responses.BaseOk:
        """
        Marks and unmarks conversations as important.
        
        :param peer_id: ID of conversation to mark as important.
        :param important: '1' — to add a star (mark as important), '0' — to remove the star
        :param group_id: Group ID (for group messages with group access token)
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'messages.markAsImportantConversation'
        response_type = responses.BaseOk
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def mark_as_read(self, message_ids: Optional[List[int]] = None, peer_id: Optional[int] = None, start_message_id: Optional[int] = None, group_id: Optional[int] = None, mark_conversation_as_read: Optional[bool] = None) -> responses.BaseOk:
        """
        Marks messages as read.
        
        :param message_ids: IDs of messages to mark as read.
        :param peer_id: Destination ID. "For user: 'User ID', e.g. '12345'. For chat: '2000000000' + 'chat_id', e.g. '2000000001'. For community: '- community ID', e.g. '-12345'. "
        :param start_message_id: Message ID to start from.
        :param group_id: Group ID (for group messages with user access token)
        :param mark_conversation_as_read:
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'messages.markAsRead'
        response_type = responses.BaseOk
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def pin(self, peer_id: int, message_id: Optional[int] = None) -> responses.MessagesPin:
        """
        Pin a message.
        
        :param peer_id: Destination ID. "For user: 'User ID', e.g. '12345'. For chat: '2000000000' + 'Chat ID', e.g. '2000000001'. For community: '- Community ID', e.g. '-12345'. "
        :param message_id:
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'messages.pin'
        response_type = responses.MessagesPin
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def remove_chat_user(self, chat_id: int, user_id: Optional[int] = None, member_id: Optional[int] = None) -> responses.BaseOk:
        """
        Allows the current user to leave a chat or, if the current user started the chat, allows the user to remove another user from the chat.
        
        :param chat_id: Chat ID.
        :param user_id: ID of the user to be removed from the chat.
        :param member_id:
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'messages.removeChatUser'
        response_type = responses.BaseOk
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def restore(self, message_id: int, group_id: Optional[int] = None) -> responses.BaseOk:
        """
        Restores a deleted message.
        
        :param message_id: ID of a previously-deleted message to restore.
        :param group_id: Group ID (for group messages with user access token)
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'messages.restore'
        response_type = responses.BaseOk
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def search(self, q: Optional[str] = None, peer_id: Optional[int] = None, date: Optional[int] = None, preview_length: Optional[int] = None, offset: Optional[int] = None, count: Optional[int] = None, extended: Optional[bool] = None, fields: Optional[List[str]] = None, group_id: Optional[int] = None) -> responses.MessagesSearch:
        """
        Returns a list of the current user's private messages that match search criteria.
        
        :param q: Search query string.
        :param peer_id: Destination ID. "For user: 'User ID', e.g. '12345'. For chat: '2000000000' + 'chat_id', e.g. '2000000001'. For community: '- community ID', e.g. '-12345'. "
        :param date: Date to search message before in Unixtime.
        :param preview_length: Number of characters after which to truncate a previewed message. To preview the full message, specify '0'. "NOTE: Messages are not truncated by default. Messages are truncated by words."
        :param offset: Offset needed to return a specific subset of messages.
        :param count: Number of messages to return.
        :param extended:
        :param fields:
        :param group_id: Group ID (for group messages with group access token)
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'messages.search'
        response_type = responses.MessagesSearch
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def search_conversations(self, q: Optional[str] = None, count: Optional[int] = None, extended: Optional[bool] = None, fields: Optional[List[Union[objects.UsersFields, str]]] = None, group_id: Optional[int] = None) -> responses.MessagesSearchConversations:
        """
        Returns a list of the current user's conversations that match search criteria.
        
        :param q: Search query string.
        :param count: Maximum number of results.
        :param extended: '1' — return extra information about users and communities
        :param fields: Profile fields to return.
        :param group_id: Group ID (for group messages with user access token)
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'messages.searchConversations'
        response_type = responses.MessagesSearchConversations
        return self._call(method_name, method_parameters, param_aliases, response_type)

    @overload
    def send(self, user_id: Optional[int] = None, random_id: Optional[int] = None, peer_id: Optional[int] = None, domain: Optional[str] = None, chat_id: Optional[int] = None, user_ids: None = None, message: Optional[str] = None, lat: Optional[float] = None, long: Optional[float] = None, attachment: Optional[str] = None, reply_to: Optional[int] = None, forward_messages: Optional[List[int]] = None, sticker_id: Optional[int] = None, group_id: Optional[int] = None, keyboard: Optional[str] = None, payload: Optional[str] = None, dont_parse_links: Optional[bool] = None, disable_mentions: Optional[bool] = None, intent: Optional[str] = None, subscribe_id: Optional[int] = None) -> responses.MessagesSend: ...
    @overload
    def send(self, user_ids: List[int], user_id: Optional[int] = None, random_id: Optional[int] = None, peer_id: Optional[int] = None, domain: Optional[str] = None, chat_id: Optional[int] = None, message: Optional[str] = None, lat: Optional[float] = None, long: Optional[float] = None, attachment: Optional[str] = None, reply_to: Optional[int] = None, forward_messages: Optional[List[int]] = None, sticker_id: Optional[int] = None, group_id: Optional[int] = None, keyboard: Optional[str] = None, payload: Optional[str] = None, dont_parse_links: Optional[bool] = None, disable_mentions: Optional[bool] = None, intent: Optional[str] = None, subscribe_id: Optional[int] = None) -> responses.MessagesSendUserIds: ...
    def send(self, user_id: Optional[int] = None, random_id: Optional[int] = None, peer_id: Optional[int] = None, domain: Optional[str] = None, chat_id: Optional[int] = None, user_ids: Optional[List[int]] = None, message: Optional[str] = None, lat: Optional[float] = None, long: Optional[float] = None, attachment: Optional[str] = None, reply_to: Optional[int] = None, forward_messages: Optional[List[int]] = None, sticker_id: Optional[int] = None, group_id: Optional[int] = None, keyboard: Optional[str] = None, payload: Optional[str] = None, dont_parse_links: Optional[bool] = None, disable_mentions: Optional[bool] = None, intent: Optional[str] = None, subscribe_id: Optional[int] = None):
        """
        Sends a message.
        
        :param user_id: User ID (by default — current user).
        :param random_id: Unique identifier to avoid resending the message.
        :param peer_id: Destination ID. "For user: 'User ID', e.g. '12345'. For chat: '2000000000' + 'chat_id', e.g. '2000000001'. For community: '- community ID', e.g. '-12345'. "
        :param domain: User's short address (for example, 'illarionov').
        :param chat_id: ID of conversation the message will relate to.
        :param user_ids: IDs of message recipients (if new conversation shall be started).
        :param message: (Required if 'attachments' is not set.) Text of the message.
        :param lat: Geographical latitude of a check-in, in degrees (from -90 to 90).
        :param long: Geographical longitude of a check-in, in degrees (from -180 to 180).
        :param attachment: (Required if 'message' is not set.) List of objects attached to the message, separated by commas, in the following format: "<owner_id>_<media_id>", '' — Type of media attachment: 'photo' — photo, 'video' — video, 'audio' — audio, 'doc' — document, 'wall' — wall post, '<owner_id>' — ID of the media attachment owner. '<media_id>' — media attachment ID. Example: "photo100172_166443618"
        :param reply_to:
        :param forward_messages: ID of forwarded messages, separated with a comma. Listed messages of the sender will be shown in the message body at the recipient's. Example: "123,431,544"
        :param sticker_id: Sticker id.
        :param group_id: Group ID (for group messages with group access token)
        :param keyboard:
        :param payload:
        :param dont_parse_links:
        :param disable_mentions:
        :param intent:
        :param subscribe_id:
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'messages.send'
        if not user_ids:
            response_type = responses.MessagesSend
        if user_ids:
            response_type = responses.MessagesSendUserIds
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def send_message_event_answer(self, event_id: str, user_id: int, peer_id: int, event_data: Optional[str] = None) -> responses.BaseOk:
        """
        :param event_id:
        :param user_id:
        :param peer_id:
        :param event_data:
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'messages.sendMessageEventAnswer'
        response_type = responses.BaseOk
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def set_activity(self, user_id: Optional[int] = None, type_: Optional[str] = None, peer_id: Optional[int] = None, group_id: Optional[int] = None) -> responses.BaseOk:
        """
        Changes the status of a user as typing in a conversation.
        
        :param user_id: User ID.
        :param type_: 'typing' — user has started to type.
        :param peer_id: Destination ID. "For user: 'User ID', e.g. '12345'. For chat: '2000000000' + 'chat_id', e.g. '2000000001'. For community: '- community ID', e.g. '-12345'. "
        :param group_id: Group ID (for group messages with group access token)
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = [('type_', 'type')]
        method_name = 'messages.setActivity'
        response_type = responses.BaseOk
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def set_chat_photo(self, file: str) -> responses.MessagesSetChatPhoto:
        """
        Sets a previously-uploaded picture as the cover picture of a chat.
        
        :param file: Upload URL from the 'response' field returned by the [vk.com/dev/photos.getChatUploadServer|photos.getChatUploadServer] method upon successfully uploading an image.
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'messages.setChatPhoto'
        response_type = responses.MessagesSetChatPhoto
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def unpin(self, peer_id: int, group_id: Optional[int] = None) -> responses.BaseOk:
        """
        :param peer_id:
        :param group_id:
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'messages.unpin'
        response_type = responses.BaseOk
        return self._call(method_name, method_parameters, param_aliases, response_type)



from . import api
import requests
from ctypes import c_longdouble


class User:
    def __init__(self, access_token: str, v='5.120', session=None, delay=0.35,
                 auto_retry=True, max_retries=2, timeout=5):
        last_call_timer = c_longdouble(0)
        if session is None:
            session = requests.session()

        self.search = api.Search(session, access_token, v, last_call_timer, delay, auto_retry, max_retries, timeout)
        self.board = api.Board(session, access_token, v, last_call_timer, delay, auto_retry, max_retries, timeout)
        self.market = api.Market(session, access_token, v, last_call_timer, delay, auto_retry, max_retries, timeout)
        self.users = api.Users(session, access_token, v, last_call_timer, delay, auto_retry, max_retries, timeout)
        self.gifts = api.Gifts(session, access_token, v, last_call_timer, delay, auto_retry, max_retries, timeout)
        self.docs = api.Docs(session, access_token, v, last_call_timer, delay, auto_retry, max_retries, timeout)
        self.stories = api.Stories(session, access_token, v, last_call_timer, delay, auto_retry, max_retries, timeout)
        self.stats = api.Stats(session, access_token, v, last_call_timer, delay, auto_retry, max_retries, timeout)
        self.wall = api.Wall(session, access_token, v, last_call_timer, delay, auto_retry, max_retries, timeout)
        self.ads = api.Ads(session, access_token, v, last_call_timer, delay, auto_retry, max_retries, timeout)
        self.utils = api.Utils(session, access_token, v, last_call_timer, delay, auto_retry, max_retries, timeout)
        self.newsfeed = api.Newsfeed(session, access_token, v, last_call_timer, delay, auto_retry, max_retries, timeout)
        self.status = api.Status(session, access_token, v, last_call_timer, delay, auto_retry, max_retries, timeout)
        self.notes = api.Notes(session, access_token, v, last_call_timer, delay, auto_retry, max_retries, timeout)
        self.pages = api.Pages(session, access_token, v, last_call_timer, delay, auto_retry, max_retries, timeout)
        self.storage = api.Storage(session, access_token, v, last_call_timer, delay, auto_retry, max_retries, timeout)
        self.account = api.Account(session, access_token, v, last_call_timer, delay, auto_retry, max_retries, timeout)
        self.orders = api.Orders(session, access_token, v, last_call_timer, delay, auto_retry, max_retries, timeout)
        self.auth = api.Auth(session, access_token, v, last_call_timer, delay, auto_retry, max_retries, timeout)
        self.database = api.Database(session, access_token, v, last_call_timer, delay, auto_retry, max_retries, timeout)
        self.streaming = api.Streaming(session, access_token, v, last_call_timer, delay, auto_retry, max_retries,
                                       timeout)
        self.likes = api.Likes(session, access_token, v, last_call_timer, delay, auto_retry, max_retries, timeout)
        self.widgets = api.Widgets(session, access_token, v, last_call_timer, delay, auto_retry, max_retries, timeout)
        self.apps = api.Apps(session, access_token, v, last_call_timer, delay, auto_retry, max_retries, timeout)
        self.downloadedGames = api.Downloadedgames(session, access_token, v, last_call_timer, delay, auto_retry,
                                                   max_retries, timeout)
        self.groups = api.Groups(session, access_token, v, last_call_timer, delay, auto_retry, max_retries, timeout)
        self.messages = api.Messages(session, access_token, v, last_call_timer, delay, auto_retry, max_retries, timeout)
        self.notifications = api.Notifications(session, access_token, v, last_call_timer, delay, auto_retry,
                                               max_retries, timeout)
        self.fave = api.Fave(session, access_token, v, last_call_timer, delay, auto_retry, max_retries, timeout)
        self.photos = api.Photos(session, access_token, v, last_call_timer, delay, auto_retry, max_retries, timeout)
        self.friends = api.Friends(session, access_token, v, last_call_timer, delay, auto_retry, max_retries, timeout)
        self.video = api.Video(session, access_token, v, last_call_timer, delay, auto_retry, max_retries, timeout)
        self.prettyCards = api.Prettycards(session, access_token, v, last_call_timer, delay, auto_retry, max_retries,
                                           timeout)
        self.polls = api.Polls(session, access_token, v, last_call_timer, delay, auto_retry, max_retries, timeout)
        self.leads = api.Leads(session, access_token, v, last_call_timer, delay, auto_retry, max_retries, timeout)




# This file was autogenerated from vk-api json schema

from typing import List, Union, Optional, overload
from easy_vk.types import objects
from easy_vk.types import responses
from easy_vk.api_category import BaseCategory
try:
    from typing import Literal
except Exception:
    from typing_extensions import Literal


class Widgets(BaseCategory):
    def __init__(self, session, access_token: str, v: str, last_call_timer, delay: float, auto_retry: bool, max_retries: int, timeout: float):
        super().__init__(session, access_token, v, last_call_timer, delay, auto_retry, max_retries, timeout)

    def get_comments(self, widget_api_id: Optional[int] = None, url: Optional[str] = None, page_id: Optional[str] = None, order: Optional[str] = None, fields: Optional[List[Union[objects.UsersFields, str]]] = None, offset: Optional[int] = None, count: Optional[int] = None) -> responses.WidgetsGetComments:
        """
        Gets a list of comments for the page added through the [vk.com/dev/Comments|Comments widget].
        
        :param widget_api_id:
        :param url:
        :param page_id:
        :param order:
        :param fields:
        :param offset:
        :param count:
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'widgets.getComments'
        response_type = responses.WidgetsGetComments
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def get_pages(self, widget_api_id: Optional[int] = None, order: Optional[str] = None, period: Optional[str] = None, offset: Optional[int] = None, count: Optional[int] = None) -> responses.WidgetsGetPages:
        """
        Gets a list of application/site pages where the [vk.com/dev/Comments|Comments widget] or [vk.com/dev/Like|Like widget] is installed.
        
        :param widget_api_id:
        :param order:
        :param period:
        :param offset:
        :param count:
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'widgets.getPages'
        response_type = responses.WidgetsGetPages
        return self._call(method_name, method_parameters, param_aliases, response_type)



# This file was autogenerated from vk-api json schema

from typing import List, Union, Optional, overload
from easy_vk.types import objects
from easy_vk.types import responses
from easy_vk.api_category import BaseCategory
try:
    from typing import Literal
except Exception:
    from typing_extensions import Literal


class Wall(BaseCategory):
    def __init__(self, session, access_token: str, v: str, last_call_timer, delay: float, auto_retry: bool, max_retries: int, timeout: float):
        super().__init__(session, access_token, v, last_call_timer, delay, auto_retry, max_retries, timeout)

    def close_comments(self, owner_id: int, post_id: int) -> responses.BaseBool:
        """
        :param owner_id:
        :param post_id:
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'wall.closeComments'
        response_type = responses.BaseBool
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def create_comment(self, post_id: int, owner_id: Optional[int] = None, from_group: Optional[int] = None, message: Optional[str] = None, reply_to_comment: Optional[int] = None, attachments: Optional[List[str]] = None, sticker_id: Optional[int] = None, guid: Optional[str] = None) -> responses.WallCreateComment:
        """
        Adds a comment to a post on a user wall or community wall.
        
        :param post_id: Post ID.
        :param owner_id: User ID or community ID. Use a negative value to designate a community ID.
        :param from_group: Group ID.
        :param message: (Required if 'attachments' is not set.) Text of the comment.
        :param reply_to_comment: ID of comment to reply.
        :param attachments: (Required if 'message' is not set.) List of media objects attached to the comment, in the following format: "<owner_id>_<media_id>,<owner_id>_<media_id>", '' — Type of media ojbect: 'photo' — photo, 'video' — video, 'audio' — audio, 'doc' — document, '<owner_id>' — ID of the media owner. '<media_id>' — Media ID. For example: "photo100172_166443618,photo66748_265827614"
        :param sticker_id: Sticker ID.
        :param guid: Unique identifier to avoid repeated comments.
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'wall.createComment'
        response_type = responses.WallCreateComment
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def delete(self, owner_id: Optional[int] = None, post_id: Optional[int] = None) -> responses.BaseOk:
        """
        Deletes a post from a user wall or community wall.
        
        :param owner_id: User ID or community ID. Use a negative value to designate a community ID.
        :param post_id: ID of the post to be deleted.
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'wall.delete'
        response_type = responses.BaseOk
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def delete_comment(self, comment_id: int, owner_id: Optional[int] = None) -> responses.BaseOk:
        """
        Deletes a comment on a post on a user wall or community wall.
        
        :param comment_id: Comment ID.
        :param owner_id: User ID or community ID. Use a negative value to designate a community ID.
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'wall.deleteComment'
        response_type = responses.BaseOk
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def edit(self, post_id: int, owner_id: Optional[int] = None, friends_only: Optional[bool] = None, message: Optional[str] = None, attachments: Optional[List[str]] = None, services: Optional[str] = None, signed: Optional[bool] = None, publish_date: Optional[int] = None, lat: Optional[float] = None, long: Optional[float] = None, place_id: Optional[int] = None, mark_as_ads: Optional[bool] = None, close_comments: Optional[bool] = None, poster_bkg_id: Optional[int] = None, poster_bkg_owner_id: Optional[int] = None, poster_bkg_access_hash: Optional[str] = None, copyright_: Optional[str] = None) -> responses.WallEdit:
        """
        Edits a post on a user wall or community wall.
        
        :param post_id:
        :param owner_id: User ID or community ID. Use a negative value to designate a community ID.
        :param friends_only:
        :param message: (Required if 'attachments' is not set.) Text of the post.
        :param attachments: (Required if 'message' is not set.) List of objects attached to the post, in the following format: "<owner_id>_<media_id>,<owner_id>_<media_id>", '' — Type of media attachment: 'photo' — photo, 'video' — video, 'audio' — audio, 'doc' — document, '<owner_id>' — ID of the media application owner. '<media_id>' — Media application ID. Example: "photo100172_166443618,photo66748_265827614", May contain a link to an external page to include in the post. Example: "photo66748_265827614,http://habrahabr.ru", "NOTE: If more than one link is being attached, an error is thrown."
        :param services:
        :param signed:
        :param publish_date:
        :param lat:
        :param long:
        :param place_id:
        :param mark_as_ads:
        :param close_comments:
        :param poster_bkg_id:
        :param poster_bkg_owner_id:
        :param poster_bkg_access_hash:
        :param copyright_:
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = [('copyright_', 'copyright')]
        method_name = 'wall.edit'
        response_type = responses.WallEdit
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def edit_ads_stealth(self, post_id: int, owner_id: Optional[int] = None, message: Optional[str] = None, attachments: Optional[List[str]] = None, signed: Optional[bool] = None, lat: Optional[float] = None, long: Optional[float] = None, place_id: Optional[int] = None, link_button: Optional[str] = None, link_title: Optional[str] = None, link_image: Optional[str] = None, link_video: Optional[str] = None) -> responses.BaseOk:
        """
        Allows to edit hidden post.
        
        :param post_id: Post ID. Used for publishing of scheduled and suggested posts.
        :param owner_id: User ID or community ID. Use a negative value to designate a community ID.
        :param message: (Required if 'attachments' is not set.) Text of the post.
        :param attachments: (Required if 'message' is not set.) List of objects attached to the post, in the following format: "<owner_id>_<media_id>,<owner_id>_<media_id>", '' — Type of media attachment: 'photo' — photo, 'video' — video, 'audio' — audio, 'doc' — document, 'page' — wiki-page, 'note' — note, 'poll' — poll, 'album' — photo album, '<owner_id>' — ID of the media application owner. '<media_id>' — Media application ID. Example: "photo100172_166443618,photo66748_265827614", May contain a link to an external page to include in the post. Example: "photo66748_265827614,http://habrahabr.ru", "NOTE: If more than one link is being attached, an error will be thrown."
        :param signed: Only for posts in communities with 'from_group' set to '1': '1' — post will be signed with the name of the posting user, '0' — post will not be signed (default)
        :param lat: Geographical latitude of a check-in, in degrees (from -90 to 90).
        :param long: Geographical longitude of a check-in, in degrees (from -180 to 180).
        :param place_id: ID of the location where the user was tagged.
        :param link_button: Link button ID
        :param link_title: Link title
        :param link_image: Link image url
        :param link_video: Link video ID in format "<owner_id>_<media_id>"
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'wall.editAdsStealth'
        response_type = responses.BaseOk
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def edit_comment(self, comment_id: int, owner_id: Optional[int] = None, message: Optional[str] = None, attachments: Optional[List[str]] = None) -> responses.BaseOk:
        """
        Edits a comment on a user wall or community wall.
        
        :param comment_id: Comment ID.
        :param owner_id: User ID or community ID. Use a negative value to designate a community ID.
        :param message: New comment text.
        :param attachments: List of objects attached to the comment, in the following format: , "<owner_id>_<media_id>,<owner_id>_<media_id>", '' — Type of media attachment: 'photo' — photo, 'video' — video, 'audio' — audio, 'doc' — document, '<owner_id>' — ID of the media attachment owner. '<media_id>' — Media attachment ID. For example: "photo100172_166443618,photo66748_265827614"
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'wall.editComment'
        response_type = responses.BaseOk
        return self._call(method_name, method_parameters, param_aliases, response_type)

    @overload
    def get(self, owner_id: Optional[int] = None, domain: Optional[str] = None, offset: Optional[int] = None, count: Optional[int] = None, filter_: Optional[str] = None, extended: None = None, fields: Optional[List[Union[objects.BaseUserGroupFields, str]]] = None) -> responses.WallGet: ...
    @overload
    def get(self, extended: bool, owner_id: Optional[int] = None, domain: Optional[str] = None, offset: Optional[int] = None, count: Optional[int] = None, filter_: Optional[str] = None, fields: Optional[List[Union[objects.BaseUserGroupFields, str]]] = None) -> responses.WallGetExtended: ...
    def get(self, owner_id: Optional[int] = None, domain: Optional[str] = None, offset: Optional[int] = None, count: Optional[int] = None, filter_: Optional[str] = None, extended: Optional[bool] = None, fields: Optional[List[Union[objects.BaseUserGroupFields, str]]] = None):
        """
        Returns a list of posts on a user wall or community wall.
        
        :param owner_id: ID of the user or community that owns the wall. By default, current user ID. Use a negative value to designate a community ID.
        :param domain: User or community short address.
        :param offset: Offset needed to return a specific subset of posts.
        :param count: Number of posts to return (maximum 100).
        :param filter_: Filter to apply: 'owner' — posts by the wall owner, 'others' — posts by someone else, 'all' — posts by the wall owner and others (default), 'postponed' — timed posts (only available for calls with an 'access_token'), 'suggests' — suggested posts on a community wall
        :param extended: '1' — to return 'wall', 'profiles', and 'groups' fields, '0' — to return no additional fields (default)
        :param fields:
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = [('filter_', 'filter')]
        method_name = 'wall.get'
        if not extended:
            response_type = responses.WallGet
        if extended:
            response_type = responses.WallGetExtended
        return self._call(method_name, method_parameters, param_aliases, response_type)

    @overload
    def get_by_id(self, posts: List[str], extended: None = None, copy_history_depth: Optional[int] = None, fields: Optional[List[Union[objects.BaseUserGroupFields, str]]] = None) -> responses.WallGetById: ...
    @overload
    def get_by_id(self, posts: List[str], extended: bool, copy_history_depth: Optional[int] = None, fields: Optional[List[Union[objects.BaseUserGroupFields, str]]] = None) -> responses.WallGetByIdExtended: ...
    def get_by_id(self, posts: List[str], extended: Optional[bool] = None, copy_history_depth: Optional[int] = None, fields: Optional[List[Union[objects.BaseUserGroupFields, str]]] = None):
        """
        Returns a list of posts from user or community walls by their IDs.
        
        :param posts: User or community IDs and post IDs, separated by underscores. Use a negative value to designate a community ID. Example: "93388_21539,93388_20904,2943_4276,-1_1"
        :param extended: '1' — to return user and community objects needed to display posts, '0' — no additional fields are returned (default)
        :param copy_history_depth: Sets the number of parent elements to include in the array 'copy_history' that is returned if the post is a repost from another wall.
        :param fields:
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'wall.getById'
        if not extended:
            response_type = responses.WallGetById
        if extended:
            response_type = responses.WallGetByIdExtended
        return self._call(method_name, method_parameters, param_aliases, response_type)

    @overload
    def get_comment(self, comment_id: int, owner_id: Optional[int] = None, extended: None = None, fields: Optional[List[Union[objects.BaseUserGroupFields, str]]] = None) -> responses.WallGetComment: ...
    @overload
    def get_comment(self, comment_id: int, extended: bool, owner_id: Optional[int] = None, fields: Optional[List[Union[objects.BaseUserGroupFields, str]]] = None) -> responses.WallGetCommentExtended: ...
    def get_comment(self, comment_id: int, owner_id: Optional[int] = None, extended: Optional[bool] = None, fields: Optional[List[Union[objects.BaseUserGroupFields, str]]] = None):
        """
        Returns a comment on a post on a user wall or community wall.
        
        :param comment_id: Comment ID.
        :param owner_id: User ID or community ID. Use a negative value to designate a community ID.
        :param extended:
        :param fields:
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'wall.getComment'
        if not extended:
            response_type = responses.WallGetComment
        if extended:
            response_type = responses.WallGetCommentExtended
        return self._call(method_name, method_parameters, param_aliases, response_type)

    @overload
    def get_comments(self, owner_id: Optional[int] = None, post_id: Optional[int] = None, need_likes: Optional[bool] = None, start_comment_id: Optional[int] = None, offset: Optional[int] = None, count: Optional[int] = None, sort: Optional[str] = None, preview_length: Optional[int] = None, extended: None = None, fields: Optional[List[Union[objects.BaseUserGroupFields, str]]] = None, comment_id: Optional[int] = None, thread_items_count: Optional[int] = None) -> responses.WallGetComments: ...
    @overload
    def get_comments(self, extended: bool, owner_id: Optional[int] = None, post_id: Optional[int] = None, need_likes: Optional[bool] = None, start_comment_id: Optional[int] = None, offset: Optional[int] = None, count: Optional[int] = None, sort: Optional[str] = None, preview_length: Optional[int] = None, fields: Optional[List[Union[objects.BaseUserGroupFields, str]]] = None, comment_id: Optional[int] = None, thread_items_count: Optional[int] = None) -> responses.WallGetCommentsExtended: ...
    def get_comments(self, owner_id: Optional[int] = None, post_id: Optional[int] = None, need_likes: Optional[bool] = None, start_comment_id: Optional[int] = None, offset: Optional[int] = None, count: Optional[int] = None, sort: Optional[str] = None, preview_length: Optional[int] = None, extended: Optional[bool] = None, fields: Optional[List[Union[objects.BaseUserGroupFields, str]]] = None, comment_id: Optional[int] = None, thread_items_count: Optional[int] = None):
        """
        Returns a list of comments on a post on a user wall or community wall.
        
        :param owner_id: User ID or community ID. Use a negative value to designate a community ID.
        :param post_id: Post ID.
        :param need_likes: '1' — to return the 'likes' field, '0' — not to return the 'likes' field (default)
        :param start_comment_id:
        :param offset: Offset needed to return a specific subset of comments.
        :param count: Number of comments to return (maximum 100).
        :param sort: Sort order: 'asc' — chronological, 'desc' — reverse chronological
        :param preview_length: Number of characters at which to truncate comments when previewed. By default, '90'. Specify '0' if you do not want to truncate comments.
        :param extended:
        :param fields:
        :param comment_id: Comment ID.
        :param thread_items_count: Count items in threads.
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'wall.getComments'
        if not extended:
            response_type = responses.WallGetComments
        if extended:
            response_type = responses.WallGetCommentsExtended
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def get_reposts(self, owner_id: Optional[int] = None, post_id: Optional[int] = None, offset: Optional[int] = None, count: Optional[int] = None) -> responses.WallGetReposts:
        """
        Returns information about reposts of a post on user wall or community wall.
        
        :param owner_id: User ID or community ID. By default, current user ID. Use a negative value to designate a community ID.
        :param post_id: Post ID.
        :param offset: Offset needed to return a specific subset of reposts.
        :param count: Number of reposts to return.
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'wall.getReposts'
        response_type = responses.WallGetReposts
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def open_comments(self, owner_id: int, post_id: int) -> responses.BaseBool:
        """
        :param owner_id:
        :param post_id:
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'wall.openComments'
        response_type = responses.BaseBool
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def pin(self, post_id: int, owner_id: Optional[int] = None) -> responses.BaseOk:
        """
        Pins the post on wall.
        
        :param post_id: Post ID.
        :param owner_id: ID of the user or community that owns the wall. By default, current user ID. Use a negative value to designate a community ID.
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'wall.pin'
        response_type = responses.BaseOk
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def post(self, owner_id: Optional[int] = None, friends_only: Optional[bool] = None, from_group: Optional[bool] = None, message: Optional[str] = None, attachments: Optional[List[str]] = None, services: Optional[str] = None, signed: Optional[bool] = None, publish_date: Optional[int] = None, lat: Optional[float] = None, long: Optional[float] = None, place_id: Optional[int] = None, post_id: Optional[int] = None, guid: Optional[str] = None, mark_as_ads: Optional[bool] = None, close_comments: Optional[bool] = None, mute_notifications: Optional[bool] = None, copyright_: Optional[str] = None) -> responses.WallPost:
        """
        Adds a new post on a user wall or community wall. Can also be used to publish suggested or scheduled posts.
        
        :param owner_id: User ID or community ID. Use a negative value to designate a community ID.
        :param friends_only: '1' — post will be available to friends only, '0' — post will be available to all users (default)
        :param from_group: For a community: '1' — post will be published by the community, '0' — post will be published by the user (default)
        :param message: (Required if 'attachments' is not set.) Text of the post.
        :param attachments: (Required if 'message' is not set.) List of objects attached to the post, in the following format: "<owner_id>_<media_id>,<owner_id>_<media_id>", '' — Type of media attachment: 'photo' — photo, 'video' — video, 'audio' — audio, 'doc' — document, 'page' — wiki-page, 'note' — note, 'poll' — poll, 'album' — photo album, '<owner_id>' — ID of the media application owner. '<media_id>' — Media application ID. Example: "photo100172_166443618,photo66748_265827614", May contain a link to an external page to include in the post. Example: "photo66748_265827614,http://habrahabr.ru", "NOTE: If more than one link is being attached, an error will be thrown."
        :param services: List of services or websites the update will be exported to, if the user has so requested. Sample values: 'twitter', 'facebook'.
        :param signed: Only for posts in communities with 'from_group' set to '1': '1' — post will be signed with the name of the posting user, '0' — post will not be signed (default)
        :param publish_date: Publication date (in Unix time). If used, posting will be delayed until the set time.
        :param lat: Geographical latitude of a check-in, in degrees (from -90 to 90).
        :param long: Geographical longitude of a check-in, in degrees (from -180 to 180).
        :param place_id: ID of the location where the user was tagged.
        :param post_id: Post ID. Used for publishing of scheduled and suggested posts.
        :param guid:
        :param mark_as_ads:
        :param close_comments:
        :param mute_notifications:
        :param copyright_:
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = [('copyright_', 'copyright')]
        method_name = 'wall.post'
        response_type = responses.WallPost
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def post_ads_stealth(self, owner_id: int, message: Optional[str] = None, attachments: Optional[List[str]] = None, signed: Optional[bool] = None, lat: Optional[float] = None, long: Optional[float] = None, place_id: Optional[int] = None, guid: Optional[str] = None, link_button: Optional[str] = None, link_title: Optional[str] = None, link_image: Optional[str] = None, link_video: Optional[str] = None) -> responses.WallPostAdsStealth:
        """
        Allows to create hidden post which will not be shown on the community's wall and can be used for creating an ad with type "Community post".
        
        :param owner_id: User ID or community ID. Use a negative value to designate a community ID.
        :param message: (Required if 'attachments' is not set.) Text of the post.
        :param attachments: (Required if 'message' is not set.) List of objects attached to the post, in the following format: "<owner_id>_<media_id>,<owner_id>_<media_id>", '' — Type of media attachment: 'photo' — photo, 'video' — video, 'audio' — audio, 'doc' — document, 'page' — wiki-page, 'note' — note, 'poll' — poll, 'album' — photo album, '<owner_id>' — ID of the media application owner. '<media_id>' — Media application ID. Example: "photo100172_166443618,photo66748_265827614", May contain a link to an external page to include in the post. Example: "photo66748_265827614,http://habrahabr.ru", "NOTE: If more than one link is being attached, an error will be thrown."
        :param signed: Only for posts in communities with 'from_group' set to '1': '1' — post will be signed with the name of the posting user, '0' — post will not be signed (default)
        :param lat: Geographical latitude of a check-in, in degrees (from -90 to 90).
        :param long: Geographical longitude of a check-in, in degrees (from -180 to 180).
        :param place_id: ID of the location where the user was tagged.
        :param guid: Unique identifier to avoid duplication the same post.
        :param link_button: Link button ID
        :param link_title: Link title
        :param link_image: Link image url
        :param link_video: Link video ID in format "<owner_id>_<media_id>"
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'wall.postAdsStealth'
        response_type = responses.WallPostAdsStealth
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def report_comment(self, owner_id: int, comment_id: int, reason: Optional[int] = None) -> responses.BaseOk:
        """
        Reports (submits a complaint about) a comment on a post on a user wall or community wall.
        
        :param owner_id: ID of the user or community that owns the wall.
        :param comment_id: Comment ID.
        :param reason: Reason for the complaint: '0' – spam, '1' – child pornography, '2' – extremism, '3' – violence, '4' – drug propaganda, '5' – adult material, '6' – insult, abuse
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'wall.reportComment'
        response_type = responses.BaseOk
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def report_post(self, owner_id: int, post_id: int, reason: Optional[int] = None) -> responses.BaseOk:
        """
        Reports (submits a complaint about) a post on a user wall or community wall.
        
        :param owner_id: ID of the user or community that owns the wall.
        :param post_id: Post ID.
        :param reason: Reason for the complaint: '0' – spam, '1' – child pornography, '2' – extremism, '3' – violence, '4' – drug propaganda, '5' – adult material, '6' – insult, abuse
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'wall.reportPost'
        response_type = responses.BaseOk
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def repost(self, object_: str, message: Optional[str] = None, group_id: Optional[int] = None, mark_as_ads: Optional[bool] = None, mute_notifications: Optional[bool] = None) -> responses.WallRepost:
        """
        Reposts (copies) an object to a user wall or community wall.
        
        :param object_: ID of the object to be reposted on the wall. Example: "wall66748_3675"
        :param message: Comment to be added along with the reposted object.
        :param group_id: Target community ID when reposting to a community.
        :param mark_as_ads:
        :param mute_notifications:
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = [('object_', 'object')]
        method_name = 'wall.repost'
        response_type = responses.WallRepost
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def restore(self, owner_id: Optional[int] = None, post_id: Optional[int] = None) -> responses.BaseOk:
        """
        Restores a post deleted from a user wall or community wall.
        
        :param owner_id: User ID or community ID from whose wall the post was deleted. Use a negative value to designate a community ID.
        :param post_id: ID of the post to be restored.
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'wall.restore'
        response_type = responses.BaseOk
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def restore_comment(self, comment_id: int, owner_id: Optional[int] = None) -> responses.BaseOk:
        """
        Restores a comment deleted from a user wall or community wall.
        
        :param comment_id: Comment ID.
        :param owner_id: User ID or community ID. Use a negative value to designate a community ID.
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'wall.restoreComment'
        response_type = responses.BaseOk
        return self._call(method_name, method_parameters, param_aliases, response_type)

    @overload
    def search(self, owner_id: Optional[int] = None, domain: Optional[str] = None, query: Optional[str] = None, owners_only: Optional[bool] = None, count: Optional[int] = None, offset: Optional[int] = None, extended: None = None, fields: Optional[List[Union[objects.BaseUserGroupFields, str]]] = None) -> responses.WallSearch: ...
    @overload
    def search(self, extended: bool, owner_id: Optional[int] = None, domain: Optional[str] = None, query: Optional[str] = None, owners_only: Optional[bool] = None, count: Optional[int] = None, offset: Optional[int] = None, fields: Optional[List[Union[objects.BaseUserGroupFields, str]]] = None) -> responses.WallSearchExtended: ...
    def search(self, owner_id: Optional[int] = None, domain: Optional[str] = None, query: Optional[str] = None, owners_only: Optional[bool] = None, count: Optional[int] = None, offset: Optional[int] = None, extended: Optional[bool] = None, fields: Optional[List[Union[objects.BaseUserGroupFields, str]]] = None):
        """
        Allows to search posts on user or community walls.
        
        :param owner_id: user or community id. "Remember that for a community 'owner_id' must be negative."
        :param domain: user or community screen name.
        :param query: search query string.
        :param owners_only: '1' – returns only page owner's posts.
        :param count: count of posts to return.
        :param offset: Offset needed to return a specific subset of posts.
        :param extended: show extended post info.
        :param fields:
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'wall.search'
        if not extended:
            response_type = responses.WallSearch
        if extended:
            response_type = responses.WallSearchExtended
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def unpin(self, post_id: int, owner_id: Optional[int] = None) -> responses.BaseOk:
        """
        Unpins the post on wall.
        
        :param post_id: Post ID.
        :param owner_id: ID of the user or community that owns the wall. By default, current user ID. Use a negative value to designate a community ID.
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'wall.unpin'
        response_type = responses.BaseOk
        return self._call(method_name, method_parameters, param_aliases, response_type)



# This file was autogenerated from vk-api json schema

from typing import List, Union, Optional, overload
from easy_vk.types import objects
from easy_vk.types import responses
from easy_vk.api_category import BaseCategory
try:
    from typing import Literal
except Exception:
    from typing_extensions import Literal


class Stats(BaseCategory):
    def __init__(self, session, access_token: str, v: str, last_call_timer, delay: float, auto_retry: bool, max_retries: int, timeout: float):
        super().__init__(session, access_token, v, last_call_timer, delay, auto_retry, max_retries, timeout)

    def get(self, group_id: Optional[int] = None, app_id: Optional[int] = None, timestamp_from: Optional[int] = None, timestamp_to: Optional[int] = None, interval: Optional[str] = None, intervals_count: Optional[int] = None, filters: Optional[List[str]] = None, stats_groups: Optional[List[str]] = None, extended: Optional[bool] = None) -> responses.StatsGet:
        """
        Returns statistics of a community or an application.
        
        :param group_id: Community ID.
        :param app_id: Application ID.
        :param timestamp_from:
        :param timestamp_to:
        :param interval:
        :param intervals_count:
        :param filters:
        :param stats_groups:
        :param extended:
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'stats.get'
        response_type = responses.StatsGet
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def get_post_reach(self, owner_id: str, post_ids: List[int]) -> responses.StatsGetPostReach:
        """
        Returns stats for a wall post.
        
        :param owner_id: post owner community id. Specify with "-" sign.
        :param post_ids: wall posts id
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'stats.getPostReach'
        response_type = responses.StatsGetPostReach
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def track_visitor(self, id_: str) -> responses.BaseOk:
        """
        :param id_:
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = [('id_', 'id')]
        method_name = 'stats.trackVisitor'
        response_type = responses.BaseOk
        return self._call(method_name, method_parameters, param_aliases, response_type)



# This file was autogenerated from vk-api json schema

from typing import List, Union, Optional, overload
from easy_vk.types import objects
from easy_vk.types import responses
from easy_vk.api_category import BaseCategory
try:
    from typing import Literal
except Exception:
    from typing_extensions import Literal


class Notifications(BaseCategory):
    def __init__(self, session, access_token: str, v: str, last_call_timer, delay: float, auto_retry: bool, max_retries: int, timeout: float):
        super().__init__(session, access_token, v, last_call_timer, delay, auto_retry, max_retries, timeout)

    def get(self, count: Optional[int] = None, start_from: Optional[str] = None, filters: Optional[List[str]] = None, start_time: Optional[int] = None, end_time: Optional[int] = None) -> responses.NotificationsGet:
        """
        Returns a list of notifications about other users' feedback to the current user's wall posts.
        
        :param count: Number of notifications to return.
        :param start_from:
        :param filters: Type of notifications to return: 'wall' — wall posts, 'mentions' — mentions in wall posts, comments, or topics, 'comments' — comments to wall posts, photos, and videos, 'likes' — likes, 'reposted' — wall posts that are copied from the current user's wall, 'followers' — new followers, 'friends' — accepted friend requests
        :param start_time: Earliest timestamp (in Unix time) of a notification to return. By default, 24 hours ago.
        :param end_time: Latest timestamp (in Unix time) of a notification to return. By default, the current time.
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'notifications.get'
        response_type = responses.NotificationsGet
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def mark_as_viewed(self) -> responses.NotificationsMarkAsViewed:
        """
        Resets the counter of new notifications about other users' feedback to the current user's wall posts.
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'notifications.markAsViewed'
        response_type = responses.NotificationsMarkAsViewed
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def send_message(self, user_ids: List[int], message: str, fragment: Optional[str] = None, group_id: Optional[int] = None, random_id: Optional[int] = None) -> responses.NotificationsSendMessage:
        """
        :param user_ids:
        :param message:
        :param fragment:
        :param group_id:
        :param random_id:
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'notifications.sendMessage'
        response_type = responses.NotificationsSendMessage
        return self._call(method_name, method_parameters, param_aliases, response_type)



# This file was autogenerated from vk-api json schema

from typing import List, Union, Optional, overload
from easy_vk.types import objects
from easy_vk.types import responses
from easy_vk.api_category import BaseCategory
try:
    from typing import Literal
except Exception:
    from typing_extensions import Literal


class Orders(BaseCategory):
    def __init__(self, session, access_token: str, v: str, last_call_timer, delay: float, auto_retry: bool, max_retries: int, timeout: float):
        super().__init__(session, access_token, v, last_call_timer, delay, auto_retry, max_retries, timeout)

    def cancel_subscription(self, user_id: int, subscription_id: int, pending_cancel: Optional[bool] = None) -> responses.OrdersCancelSubscription:
        """
        :param user_id:
        :param subscription_id:
        :param pending_cancel:
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'orders.cancelSubscription'
        response_type = responses.OrdersCancelSubscription
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def change_state(self, order_id: int, action: str, app_order_id: Optional[int] = None, test_mode: Optional[bool] = None) -> responses.OrdersChangeState:
        """
        Changes order status.
        
        :param order_id: order ID.
        :param action: action to be done with the order. Available actions: *cancel — to cancel unconfirmed order. *charge — to confirm unconfirmed order. Applies only if processing of [vk.com/dev/payments_status|order_change_state] notification failed. *refund — to cancel confirmed order.
        :param app_order_id: internal ID of the order in the application.
        :param test_mode: if this parameter is set to 1, this method returns a list of test mode orders. By default — 0.
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'orders.changeState'
        response_type = responses.OrdersChangeState
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def get(self, offset: Optional[int] = None, count: Optional[int] = None, test_mode: Optional[bool] = None) -> responses.OrdersGet:
        """
        Returns a list of orders.
        
        :param offset:
        :param count: number of returned orders.
        :param test_mode: if this parameter is set to 1, this method returns a list of test mode orders. By default — 0.
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'orders.get'
        response_type = responses.OrdersGet
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def get_amount(self, user_id: int, votes: List[str]) -> responses.OrdersGetAmount:
        """
        :param user_id:
        :param votes:
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'orders.getAmount'
        response_type = responses.OrdersGetAmount
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def get_by_id(self, order_id: Optional[int] = None, order_ids: Optional[List[int]] = None, test_mode: Optional[bool] = None) -> responses.OrdersGetById:
        """
        Returns information about orders by their IDs.
        
        :param order_id: order ID.
        :param order_ids: order IDs (when information about several orders is requested).
        :param test_mode: if this parameter is set to 1, this method returns a list of test mode orders. By default — 0.
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'orders.getById'
        response_type = responses.OrdersGetById
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def get_user_subscription_by_id(self, user_id: int, subscription_id: int) -> responses.OrdersGetUserSubscriptionById:
        """
        :param user_id:
        :param subscription_id:
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'orders.getUserSubscriptionById'
        response_type = responses.OrdersGetUserSubscriptionById
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def get_user_subscriptions(self, user_id: int) -> responses.OrdersGetUserSubscriptions:
        """
        :param user_id:
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'orders.getUserSubscriptions'
        response_type = responses.OrdersGetUserSubscriptions
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def update_subscription(self, user_id: int, subscription_id: int, price: int) -> responses.OrdersUpdateSubscription:
        """
        :param user_id:
        :param subscription_id:
        :param price:
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'orders.updateSubscription'
        response_type = responses.OrdersUpdateSubscription
        return self._call(method_name, method_parameters, param_aliases, response_type)



from typing import List
from enum import Enum
import json


class ActionType(Enum):
    TEXT = 'text'
    OPEN_LINK = 'open_link'
    LOCATION = 'location'
    VKPAY = 'vkpay'
    OPEN_APP = 'open_app'
    CALLBACK = 'callback'


class Action:
    def __init__(self, type_: ActionType, label: str = None, link: str = None, payload: str = None, app_id: int = None,
                 owner_id: int = None, hash_: str = None):

        if type_ == ActionType.TEXT:
            assert label is not None
        elif type_ == ActionType.OPEN_LINK:
            assert label is not None
            assert link is not None
        elif type_ == ActionType.LOCATION:
            pass
        elif type_ == ActionType.VKPAY:
            assert hash_ is not None
        elif type_ == ActionType.OPEN_APP:
            assert app_id is not None
            assert label is not None
        elif type_ == ActionType.CALLBACK:
            assert label is not None

        self.type = type_
        self.label = label
        self.link = link
        self.payload = payload
        self.app_id = app_id
        self.owner_id = owner_id
        self.hash = hash_

    def generate(self):
        action_dict = {k: v for k, v in self.__dict__.items() if v}
        return action_dict


class Color(Enum):
    PRIMARY = 'primary'
    SECONDARY = 'secondary'
    NEGATIVE = 'negative'
    POSITIVE = 'positive'


class Button:
    def __init__(self, action: Action, color: Color = None):
        self.action = action
        self.color = color

    def generate(self):
        button_dict = {'action': self.action.generate()}
        if self.color:
            button_dict['color'] = self.color.value
        return button_dict

    @classmethod
    def from_template(cls, tmpl: str):
        parameters = tmpl.replace('||', '').split('|')
        action_dict = {'type_': parameters[0]}
        color = None

        for p in parameters[1:]:
            key, value = p.split('::')
            if key == 'color':
                color = Color(value)
            else:
                action_dict[key] = value

        action = Action(**action_dict)
        return cls(action, color)


class Row:
    def __init__(self, buttons: List[Button]):
        self.buttons = buttons

    def add_button(self, button: Button):
        self.buttons.append(button)

    def generate(self):
        return [b.generate() for b in self.buttons]

    @classmethod
    def from_template(cls, row: str):
        buttons = [Button.from_template(b) for b in row.split('  ')]
        return cls(buttons)


class Keyboard:
    def __init__(self, one_time: bool = False, inline: bool = False, rows: List[Row] = None):
        self.one_time = one_time
        self.inline = inline
        self.rows = rows

    def to_json(self):
        """
        Returns json string of keyboard
        """

        keyboard_dict = {'buttons': [r.generate() for r in self.rows]}
        if self.one_time:
            keyboard_dict['one_time'] = True
        if self.inline:
            keyboard_dict['inline'] = True

        return json.dumps(keyboard_dict)

    @classmethod
    def from_template(cls, tmpl: str):
        """
        Creates keyboard class from keyboard template

        :param tmpl: keyboard template
        :return:
        """
        tmpl_rows = tmpl.split('\n')

        one_time = False
        inline = False

        if 'one_time' in tmpl_rows[0]:
            one_time = True
        if 'inline' in tmpl_rows[0]:
            inline = True

        rows = [Row.from_template(r) for r in tmpl_rows[2:] if r]

        return cls(one_time, inline, rows)

# template = """one_time
# ___
# ||open_link|link::https://www.youtube.com/watch?v=LMkwooZ8-dE||  ||text|label::Hello, world!|color::negative||
# ||location||
# """
#
#
# kb = Keyboard.from_template(template).to_json()
# print(kb)



# This file was autogenerated from vk-api json schema

from pydantic import BaseModel, Field
from typing import List, Union, Optional, NewType
from enum import Enum
from . import objects


class AccountChangePassword(BaseModel):
    token: str = Field(..., description="New token")
    secret: Optional[str] = Field(None, description="New secret")


class AccountGetActiveOffers(BaseModel):
    count: int = Field(..., description="Total number")
    items: List[objects.AccountOffer] = Field(...)


AccountGetAppPermissions = int


class AccountGetBanned(BaseModel):
    count: int = Field(..., description="Total number")
    items: List[int] = Field(...)
    profiles: Optional[List[objects.UsersUserMin]] = Field(None)
    groups: Optional[List[objects.GroupsGroup]] = Field(None)


class AccountGetCounters(objects.AccountAccountCounters):
    ...


class AccountGetInfo(objects.AccountInfo):
    ...


class AccountGetProfileInfo(objects.AccountUserSettings):
    ...


class AccountGetPushSettings(objects.AccountPushSettings):
    ...


class AccountSaveProfileInfo(BaseModel):
    changed: Optional[objects.BaseBoolInt] = Field(None, description="1 if changes has been processed")
    name_request: Optional[objects.AccountNameRequest] = Field(None)


AdsAddOfficeUsers = bool


class AdsCheckLink(objects.AdsLinkStatus):
    ...


AdsCreateAds = List[int]


AdsCreateCampaigns = List[int]


AdsCreateClients = List[int]


class AdsCreateTargetGroup(BaseModel):
    id_: Optional[int] = Field(None, alias='id', description="Group ID")
    pixel: Optional[str] = Field(None, description="Pixel code")


AdsDeleteAds = List[int]


AdsDeleteCampaigns = int


AdsDeleteClients = int


AdsGetAccounts = List[objects.AdsAccount]


AdsGetAdsLayout = List[objects.AdsAdLayout]


AdsGetAdsTargeting = List[objects.AdsTargSettings]


AdsGetAds = List[objects.AdsAd]


AdsGetBudget = int


AdsGetCampaigns = List[objects.AdsCampaign]


class AdsGetCategories(BaseModel):
    v1: Optional[List[objects.AdsCategory]] = Field(None, description="Old categories")
    v2: Optional[List[objects.AdsCategory]] = Field(None, description="Actual categories")


AdsGetClients = List[objects.AdsClient]


AdsGetDemographics = List[objects.AdsDemoStats]


class AdsGetFloodStats(objects.AdsFloodStats):
    ...


class AdsGetLookalikeRequests(BaseModel):
    count: int = Field(..., description="Total count of found lookalike requests")
    items: List[objects.AdsLookalikeRequest] = Field(..., description="found lookalike requests")


class AdsGetMusicians(BaseModel):
    items: List[objects.AdsMusician] = Field(..., description="Musicians")


AdsGetOfficeUsers = List[objects.AdsUsers]


AdsGetPostsReach = List[objects.AdsPromotedPostReach]


class AdsGetRejectionReason(objects.AdsRejectReason):
    ...


AdsGetStatistics = List[objects.AdsStats]


AdsGetSuggestionsCities = List[objects.AdsTargSuggestionsCities]


AdsGetSuggestionsRegions = List[objects.AdsTargSuggestionsRegions]


AdsGetSuggestions = List[objects.AdsTargSuggestions]


AdsGetSuggestionsSchools = List[objects.AdsTargSuggestionsSchools]


AdsGetTargetGroups = List[objects.AdsTargetGroup]


class AdsGetTargetingStats(objects.AdsTargStats):
    ...


AdsGetUploadUrl = str


AdsGetVideoUploadUrl = str


AdsImportTargetContacts = int


AdsRemoveOfficeUsers = bool


AdsUpdateAds = List[int]


AdsUpdateCampaigns = int


AdsUpdateClients = int


class AppsGetCatalog(BaseModel):
    count: Optional[int] = Field(None, description="Total number")
    items: Optional[List[objects.AppsApp]] = Field(None)


class AppsGetFriendsList(BaseModel):
    count: Optional[int] = Field(None, description="Total number")
    items: Optional[List[objects.UsersUserFull]] = Field(None)


class AppsGetLeaderboardExtended(BaseModel):
    count: Optional[int] = Field(None, description="Total number")
    items: Optional[List[objects.AppsLeaderboard]] = Field(None)
    profiles: Optional[List[objects.UsersUserMin]] = Field(None)


class AppsGetLeaderboard(BaseModel):
    count: Optional[int] = Field(None, description="Total number")
    items: Optional[List[objects.AppsLeaderboard]] = Field(None)


class AppsGetScopes(BaseModel):
    count: int = Field(..., description="Total number")
    items: List[objects.AppsScope] = Field(...)


AppsGetScore = int


class AppsGet(BaseModel):
    count: Optional[int] = Field(None, description="Total number of applications")
    items: Optional[List[objects.AppsApp]] = Field(None, description="List of applications")


AppsSendRequest = int


class AuthRestore(BaseModel):
    success: Optional[int] = Field(None, description="1 if success")
    sid: Optional[str] = Field(None, description="Parameter needed to grant access by code")


BaseBool = objects.BaseBoolInt


class BaseGetUploadServer(objects.BaseUploadServer):
    ...


BaseOk = int


BoardAddTopic = int


BoardCreateComment = int


class BoardGetCommentsExtended(BaseModel):
    count: int = Field(..., description="Total number")
    items: List[objects.BoardTopicComment] = Field(...)
    poll: Optional[objects.BoardTopicPoll] = Field(None)
    profiles: List[objects.UsersUser] = Field(...)
    groups: List[objects.GroupsGroup] = Field(...)


class BoardGetComments(BaseModel):
    count: int = Field(..., description="Total number")
    items: List[objects.BoardTopicComment] = Field(...)
    poll: Optional[objects.BoardTopicPoll] = Field(None)


class BoardGetTopicsExtended(BaseModel):
    count: int = Field(..., description="Total number")
    items: List[objects.BoardTopic] = Field(...)
    default_order: objects.BoardDefaultOrder = Field(...)
    can_add_topics: objects.BaseBoolInt = Field(..., description="Information whether current user can add topic")
    profiles: List[objects.UsersUserMin] = Field(...)


class BoardGetTopics(BaseModel):
    count: int = Field(..., description="Total number")
    items: List[objects.BoardTopic] = Field(...)
    default_order: objects.BoardDefaultOrder = Field(...)
    can_add_topics: objects.BaseBoolInt = Field(..., description="Information whether current user can add topic")


class DatabaseGetChairs(BaseModel):
    count: Optional[int] = Field(None, description="Total number")
    items: Optional[List[objects.BaseObject]] = Field(None)


DatabaseGetCitiesById = List[objects.BaseObject]


class DatabaseGetCities(BaseModel):
    count: Optional[int] = Field(None, description="Total number")
    items: Optional[List[objects.DatabaseCity]] = Field(None)


DatabaseGetCountriesById = List[objects.BaseCountry]


class DatabaseGetCountries(BaseModel):
    count: Optional[int] = Field(None, description="Total number")
    items: Optional[List[objects.BaseCountry]] = Field(None)


class DatabaseGetFaculties(BaseModel):
    count: Optional[int] = Field(None, description="Total number")
    items: Optional[List[objects.DatabaseFaculty]] = Field(None)


DatabaseGetMetroStationsById = List[objects.DatabaseStation]


class DatabaseGetMetroStations(BaseModel):
    count: Optional[int] = Field(None, description="Total number")
    items: Optional[List[objects.DatabaseStation]] = Field(None)


class DatabaseGetRegions(BaseModel):
    count: Optional[int] = Field(None, description="Total number")
    items: Optional[List[objects.DatabaseRegion]] = Field(None)


DatabaseGetSchoolClasses = List[List[Union[str, int]]]


class DatabaseGetSchools(BaseModel):
    count: Optional[int] = Field(None, description="Total number")
    items: Optional[List[objects.DatabaseSchool]] = Field(None)


class DatabaseGetUniversities(BaseModel):
    count: Optional[int] = Field(None, description="Total number")
    items: Optional[List[objects.DatabaseUniversity]] = Field(None)


class DocsAdd(BaseModel):
    id_: Optional[int] = Field(None, alias='id', description="Doc ID")


DocsGetById = List[objects.DocsDoc]


class DocsGetTypes(BaseModel):
    count: Optional[int] = Field(None, description="Total number")
    items: Optional[List[objects.DocsDocTypes]] = Field(None)


class DocsGetUploadServer(objects.BaseUploadServer):
    ...


class DocsGet(BaseModel):
    count: int = Field(..., description="Total number")
    items: List[objects.DocsDoc] = Field(...)


class DocsSave(BaseModel):
    type_: Optional[objects.DocsDocAttachmentType] = Field(None, alias='type')
    audio_message: Optional[objects.MessagesAudioMessage] = Field(None)
    doc: Optional[objects.DocsDoc] = Field(None)
    graffiti: Optional[objects.MessagesGraffiti] = Field(None)


class DocsSearch(BaseModel):
    count: int = Field(..., description="Total number")
    items: List[objects.DocsDoc] = Field(...)


class DownloadedGamesPaidStatus(BaseModel):
    is_paid: bool = Field(..., description="Game has been paid")


class FaveAddTag(objects.FaveTag):
    ...


class FaveGetPages(BaseModel):
    count: Optional[int] = Field(None)
    items: Optional[List[objects.FavePage]] = Field(None)


class FaveGetTags(BaseModel):
    count: Optional[int] = Field(None)
    items: Optional[List[objects.FaveTag]] = Field(None)


class FaveGetExtended(BaseModel):
    count: Optional[int] = Field(None, description="Total number")
    items: Optional[List[objects.FaveBookmark]] = Field(None)
    profiles: Optional[List[objects.UsersUserFull]] = Field(None)
    groups: Optional[List[objects.GroupsGroup]] = Field(None)


class FaveGet(BaseModel):
    count: Optional[int] = Field(None, description="Total number")
    items: Optional[List[objects.FaveBookmark]] = Field(None)


class FriendsAddList(BaseModel):
    list_id: int = Field(..., description="List ID")


FriendsAdd = int


FriendsAreFriendsExtended = List[objects.FriendsFriendExtendedStatus]


FriendsAreFriends = List[objects.FriendsFriendStatus]


class FriendsDelete(BaseModel):
    success: int = Field(...)
    friend_deleted: Optional[int] = Field(None, description="Returns 1 if friend has been deleted")
    out_request_deleted: Optional[int] = Field(None, description="Returns 1 if out request has been canceled")
    in_request_deleted: Optional[int] = Field(None, description="Returns 1 if incoming request has been declined")
    suggestion_deleted: Optional[int] = Field(None, description="Returns 1 if suggestion has been declined")


FriendsGetAppUsers = List[int]


FriendsGetByPhones = List[objects.FriendsUserXtrPhone]


class FriendsGetLists(BaseModel):
    count: int = Field(..., description="Total number of friends lists")
    items: List[objects.FriendsFriendsList] = Field(...)


FriendsGetMutual = List[int]


FriendsGetMutualTargetUids = List[objects.FriendsMutualFriend]


class FriendsGetOnlineOnlineMobile(BaseModel):
    online: Optional[List[int]] = Field(None)
    online_mobile: Optional[List[int]] = Field(None)


FriendsGetOnline = List[int]


FriendsGetRecent = List[int]


class FriendsGetRequestsExtended(BaseModel):
    count: Optional[int] = Field(None, description="Total requests number")
    items: Optional[List[objects.FriendsRequestsXtrMessage]] = Field(None)


class FriendsGetRequestsNeedMutual(BaseModel):
    count: Optional[int] = Field(None, description="Total requests number")
    items: Optional[List[objects.FriendsRequests]] = Field(None)


class FriendsGetRequests(BaseModel):
    count: Optional[int] = Field(None, description="Total requests number")
    items: Optional[List[int]] = Field(None)
    count_unread: Optional[int] = Field(None, description="Total unread requests number")


class FriendsGetSuggestions(BaseModel):
    count: int = Field(..., description="Total results number")
    items: List[objects.UsersUserFull] = Field(...)


class FriendsGetFields(BaseModel):
    count: int = Field(..., description="Total friends number")
    items: List[objects.FriendsUserXtrLists] = Field(...)


class FriendsGet(BaseModel):
    count: int = Field(..., description="Total friends number")
    items: List[int] = Field(...)


class FriendsSearch(BaseModel):
    count: int = Field(..., description="Total number")
    items: List[objects.UsersUserFull] = Field(...)


class GiftsGet(BaseModel):
    count: Optional[int] = Field(None, description="Total number")
    items: Optional[List[objects.GiftsGift]] = Field(None)


class GroupsAddAddress(objects.GroupsAddress):
    ...


class GroupsAddCallbackServer(BaseModel):
    server_id: Optional[int] = Field(None)


class GroupsAddLink(objects.GroupsGroupLink):
    ...


class GroupsCreate(objects.GroupsGroup):
    ...


class GroupsEditAddress(objects.GroupsAddress):
    ...


class GroupsGetAddresses(BaseModel):
    count: int = Field(..., description="Total count of addresses")
    items: List[objects.GroupsAddress] = Field(...)


class GroupsGetBanned(BaseModel):
    count: int = Field(..., description="Total users number")
    # items: List[objects.GroupsBannedItem] = Field(...)


GroupsGetById = List[objects.GroupsGroupFull]


class GroupsGetCallbackConfirmationCode(BaseModel):
    code: Optional[str] = Field(None, description="Confirmation code")


class GroupsGetCallbackServers(BaseModel):
    count: int = Field(...)
    items: List[objects.GroupsCallbackServer] = Field(...)


class GroupsGetCallbackSettings(objects.GroupsCallbackSettings):
    ...


class GroupsGetCatalogInfoExtended(BaseModel):
    enabled: int = Field(..., description="Information whether catalog is enabled for current user")
    categories: Optional[List[objects.GroupsGroupCategoryFull]] = Field(None)


class GroupsGetCatalogInfo(BaseModel):
    enabled: int = Field(..., description="Information whether catalog is enabled for current user")
    categories: Optional[List[objects.GroupsGroupCategory]] = Field(None)


class GroupsGetCatalog(BaseModel):
    count: int = Field(..., description="Total communities number")
    items: List[objects.GroupsGroup] = Field(...)


class GroupsGetInvitedUsers(BaseModel):
    count: int = Field(..., description="Total communities number")
    items: List[objects.UsersUserFull] = Field(...)


class GroupsGetInvitesExtended(BaseModel):
    count: int = Field(..., description="Total communities number")
    items: List[objects.GroupsGroupXtrInvitedBy] = Field(...)
    profiles: List[objects.UsersUserMin] = Field(...)
    groups: List[objects.GroupsGroupFull] = Field(...)


class GroupsGetInvites(BaseModel):
    count: int = Field(..., description="Total communities number")
    items: List[objects.GroupsGroupXtrInvitedBy] = Field(...)


class GroupsGetLongPollServer(objects.GroupsLongPollServer):
    ...


class GroupsGetLongPollSettings(objects.GroupsLongPollSettings):
    ...


class GroupsGetMembersFields(BaseModel):
    count: int = Field(..., description="Total members number")
    items: List[objects.GroupsUserXtrRole] = Field(...)


class GroupsGetMembersFilter(BaseModel):
    count: int = Field(..., description="Total members number")
    items: List[objects.GroupsMemberRole] = Field(...)


class GroupsGetMembers(BaseModel):
    count: int = Field(..., description="Total members number")
    items: List[int] = Field(...)


class GroupsGetRequestsFields(BaseModel):
    count: int = Field(..., description="Total communities number")
    items: List[objects.UsersUserFull] = Field(...)


class GroupsGetRequests(BaseModel):
    count: int = Field(..., description="Total communities number")
    items: List[int] = Field(...)


class GroupsGetSettings(BaseModel):
    access: Optional[int] = Field(None, description="Community access settings")
    address: Optional[str] = Field(None, description="Community's page domain")
    audio: int = Field(..., description="Audio settings")
    articles: int = Field(..., description="Articles settings")
    city_id: int = Field(..., description="City id of group")
    contacts: Optional[objects.BaseBoolInt] = Field(None)
    links: Optional[objects.BaseBoolInt] = Field(None)
    sections_list: Optional[dict] = Field(None)
    main_section: Optional[objects.GroupsGroupFullMainSection] = Field(None)
    secondary_section: Optional[int] = Field(None)
    age_limits: Optional[int] = Field(None)
    country_id: int = Field(..., description="Country id of group")
    description: str = Field(..., description="Community description")
    docs: int = Field(..., description="Docs settings")
    events: Optional[objects.BaseBoolInt] = Field(None)
    obscene_filter: objects.BaseBoolInt = Field(..., description="Information whether the obscene filter is enabled")
    obscene_stopwords: objects.BaseBoolInt = Field(..., description="Information whether the stopwords filter is enabled")
    obscene_words: List[str] = Field(..., description="The list of stop words")
    event_group_id: Optional[int] = Field(None)
    photos: int = Field(..., description="Photos settings")
    public_category: Optional[int] = Field(None, description="Information about the group category")
    public_category_list: Optional[List[objects.GroupsGroupPublicCategoryList]] = Field(None)
    public_date: Optional[str] = Field(None)
    public_date_label: Optional[str] = Field(None)
    public_subcategory: Optional[int] = Field(None, description="Information about the group subcategory")
    rss: Optional[str] = Field(None, description="URL of the RSS feed")
    start_date: Optional[int] = Field(None, description="Start date")
    finish_date: Optional[int] = Field(None, description="Finish date in Unixtime format")
    subject: Optional[int] = Field(None, description="Community subject ID")
    subject_list: Optional[List[objects.GroupsSubjectItem]] = Field(None)
    suggested_privacy: Optional[int] = Field(None)
    title: str = Field(..., description="Community title")
    topics: int = Field(..., description="Topics settings")
    twitter: Optional[objects.GroupsSettingsTwitter] = Field(None)
    video: int = Field(..., description="Video settings")
    wall: int = Field(..., description="Wall settings")
    website: Optional[str] = Field(None, description="Community website")
    phone: Optional[str] = Field(None, description="Community phone")
    email: Optional[str] = Field(None, description="Community email")
    wiki: int = Field(..., description="Wiki settings")


class GroupsGetTokenPermissions(BaseModel):
    mask: int = Field(...)
    permissions: List[objects.GroupsTokenPermissionSetting] = Field(...)


class GroupsGetExtended(BaseModel):
    count: int = Field(..., description="Total communities number")
    items: List[objects.GroupsGroupFull] = Field(...)


class GroupsGet(BaseModel):
    count: int = Field(..., description="Total communities number")
    items: List[int] = Field(...)


class GroupsIsMemberExtended(BaseModel):
    member: objects.BaseBoolInt = Field(..., description="Information whether user is a member of the group")
    invitation: Optional[objects.BaseBoolInt] = Field(None, description="Information whether user has been invited to the group")
    can_invite: Optional[objects.BaseBoolInt] = Field(None, description="Information whether user can be invited")
    can_recall: Optional[objects.BaseBoolInt] = Field(None, description="Information whether user's invite to the group can be recalled")
    request: Optional[objects.BaseBoolInt] = Field(None, description="Information whether user has sent request to the group")


GroupsIsMember = objects.BaseBoolInt


GroupsIsMemberUserIdsExtended = List[objects.GroupsMemberStatusFull]


GroupsIsMemberUserIds = List[objects.GroupsMemberStatus]


class GroupsSearch(BaseModel):
    count: int = Field(..., description="Total communities number")
    items: List[objects.GroupsGroup] = Field(...)


class LeadsCheckUser(objects.LeadsChecked):
    ...


class LeadsComplete(objects.LeadsComplete):
    ...


class LeadsGetStats(objects.LeadsLead):
    ...


LeadsGetUsers = List[objects.LeadsEntry]


class LeadsMetricHit(BaseModel):
    result: Optional[bool] = Field(None, description="Information whether request has been processed successfully")
    redirect_link: Optional[str] = Field(None, description="Redirect link")


class LeadsStart(objects.LeadsStart):
    ...


class LikesAdd(BaseModel):
    likes: int = Field(..., description="Total likes number")


class LikesDelete(BaseModel):
    likes: int = Field(..., description="Total likes number")


class LikesGetListExtended(BaseModel):
    count: int = Field(..., description="Total number")
    items: List[objects.UsersUserMin] = Field(...)


class LikesGetList(BaseModel):
    count: int = Field(..., description="Total number")
    items: List[int] = Field(...)


class LikesIsLiked(BaseModel):
    liked: objects.BaseBoolInt = Field(..., description="Information whether user liked the object")
    copied: objects.BaseBoolInt = Field(..., description="Information whether user reposted the object")


class MarketAddAlbum(BaseModel):
    market_album_id: Optional[int] = Field(None, description="Album ID")


class MarketAdd(BaseModel):
    market_item_id: Optional[int] = Field(None, description="Item ID")


MarketCreateComment = int


MarketDeleteComment = objects.BaseBoolInt


class MarketGetAlbumById(BaseModel):
    count: Optional[int] = Field(None, description="Total number")
    items: Optional[List[objects.MarketMarketAlbum]] = Field(None)


class MarketGetAlbums(BaseModel):
    count: Optional[int] = Field(None, description="Total number")
    items: Optional[List[objects.MarketMarketAlbum]] = Field(None)


class MarketGetByIdExtended(BaseModel):
    count: Optional[int] = Field(None, description="Total number")
    items: Optional[List[objects.MarketMarketItemFull]] = Field(None)


class MarketGetById(BaseModel):
    count: Optional[int] = Field(None, description="Total number")
    items: Optional[List[objects.MarketMarketItem]] = Field(None)


class MarketGetCategories(BaseModel):
    count: Optional[int] = Field(None, description="Total number")
    items: Optional[List[objects.MarketMarketCategory]] = Field(None)


class MarketGetComments(BaseModel):
    count: Optional[int] = Field(None, description="Total number")
    items: Optional[List[objects.WallWallComment]] = Field(None)


class MarketGetExtended(BaseModel):
    count: Optional[int] = Field(None, description="Total number")
    items: Optional[List[objects.MarketMarketItemFull]] = Field(None)


class MarketGet(BaseModel):
    count: Optional[int] = Field(None, description="Total number")
    items: Optional[List[objects.MarketMarketItem]] = Field(None)


MarketRestoreComment = objects.BaseBoolInt


class MarketSearchExtended(BaseModel):
    count: Optional[int] = Field(None, description="Total number")
    items: Optional[List[objects.MarketMarketItemFull]] = Field(None)


class MarketSearch(BaseModel):
    count: Optional[int] = Field(None, description="Total number")
    items: Optional[List[objects.MarketMarketItem]] = Field(None)


MessagesCreateChat = int


class MessagesDeleteChatPhoto(BaseModel):
    message_id: Optional[int] = Field(None, description="Service message ID")
    chat: Optional[objects.MessagesChat] = Field(None)


class MessagesDeleteConversation(BaseModel):
    last_deleted_id: int = Field(..., description="Id of the last message, that was deleted")


# response MessagesDelete has error in json-schema


MessagesEdit = objects.BaseBoolInt


class MessagesGetByConversationMessageId(BaseModel):
    count: int = Field(..., description="Total number")
    items: List[objects.MessagesMessage] = Field(...)


class MessagesGetByIdExtended(BaseModel):
    count: int = Field(..., description="Total number")
    items: List[objects.MessagesMessage] = Field(...)
    profiles: List[objects.UsersUserFull] = Field(...)
    groups: Optional[List[objects.GroupsGroupFull]] = Field(None)


class MessagesGetById(BaseModel):
    count: int = Field(..., description="Total number")
    items: List[objects.MessagesMessage] = Field(...)


class MessagesGetChatPreview(BaseModel):
    preview: Optional[objects.MessageChatPreview] = Field(None)
    profiles: Optional[List[objects.UsersUserFull]] = Field(None)


MessagesGetChatChatIdsFields = List[objects.MessagesChatFull]


MessagesGetChatChatIds = List[objects.MessagesChat]


class MessagesGetChatFields(objects.MessagesChatFull):
    ...


class MessagesGetChat(objects.MessagesChat):
    ...


class MessagesGetConversationMembers(BaseModel):
    count: int = Field(..., description="Chat members count")
    items: List[objects.MessagesConversationMember] = Field(...)
    chat_restrictions: Optional[objects.MessagesChatRestrictions] = Field(None)
    profiles: Optional[List[objects.UsersUserFull]] = Field(None)
    groups: Optional[List[objects.GroupsGroupFull]] = Field(None)


class MessagesGetConversationsByIdExtended(BaseModel):
    count: int = Field(..., description="Total number")
    items: List[objects.MessagesConversation] = Field(...)
    profiles: Optional[List[objects.UsersUser]] = Field(None)


class MessagesGetConversationsById(BaseModel):
    count: int = Field(..., description="Total number")
    items: List[objects.MessagesConversation] = Field(...)


class MessagesGetConversations(BaseModel):
    count: int = Field(..., description="Total number")
    unread_count: Optional[int] = Field(None, description="Unread dialogs number")
    items: List[objects.MessagesConversationWithMessage] = Field(...)
    profiles: Optional[List[objects.UsersUserFull]] = Field(None)
    groups: Optional[List[objects.GroupsGroupFull]] = Field(None)


class MessagesGetHistoryAttachments(BaseModel):
    items: Optional[List[objects.MessagesHistoryAttachment]] = Field(None)
    next_from: Optional[str] = Field(None, description="Value for pagination")


class MessagesGetHistory(BaseModel):
    count: int = Field(..., description="Total number")
    items: List[objects.MessagesMessage] = Field(...)
    profiles: Optional[List[objects.UsersUserFull]] = Field(None)
    groups: Optional[List[objects.GroupsGroupFull]] = Field(None)


class MessagesGetInviteLink(BaseModel):
    link: Optional[str] = Field(None)


class MessagesGetLastActivity(objects.MessagesLastActivity):
    ...


class MessagesGetLongPollHistory(BaseModel):
    history: Optional[List[List[int]]] = Field(None)
    groups: Optional[List[objects.GroupsGroup]] = Field(None)
    messages: Optional[objects.MessagesLongpollMessages] = Field(None)
    profiles: Optional[List[objects.UsersUserFull]] = Field(None)
    chats: Optional[List[objects.MessagesChat]] = Field(None)
    new_pts: Optional[int] = Field(None, description="Persistence timestamp")
    more: Optional[bool] = Field(None, description="Has more")
    conversations: Optional[List[objects.MessagesConversation]] = Field(None)


class MessagesGetLongPollServer(objects.MessagesLongpollParams):
    ...


class MessagesIsMessagesFromGroupAllowed(BaseModel):
    is_allowed: Optional[objects.BaseBoolInt] = Field(None)


class MessagesJoinChatByInviteLink(BaseModel):
    chat_id: Optional[int] = Field(None)


MessagesMarkAsImportant = List[int]


class MessagesPin(objects.MessagesPinnedMessage):
    ...


class MessagesSearchConversations(BaseModel):
    count: Optional[int] = Field(None, description="Total results number")
    items: Optional[List[objects.MessagesConversation]] = Field(None)
    profiles: Optional[List[objects.UsersUserFull]] = Field(None)
    groups: Optional[List[objects.GroupsGroupFull]] = Field(None)


class MessagesSearch(BaseModel):
    count: int = Field(..., description="Total number")
    items: List[objects.MessagesMessage] = Field(...)


MessagesSend = int


MessagesSendUserIds = List[dict]


class MessagesSetChatPhoto(BaseModel):
    message_id: Optional[int] = Field(None, description="Service message ID")
    chat: Optional[objects.MessagesChat] = Field(None)


class NewsfeedGetBannedExtended(BaseModel):
    groups: Optional[List[objects.UsersUserFull]] = Field(None)
    profiles: Optional[List[objects.GroupsGroupFull]] = Field(None)


class NewsfeedGetBanned(BaseModel):
    groups: Optional[List[int]] = Field(None)
    members: Optional[List[int]] = Field(None)


class NewsfeedGetComments(BaseModel):
    items: List[objects.NewsfeedNewsfeedItem] = Field(...)
    profiles: List[objects.UsersUserFull] = Field(...)
    groups: List[objects.GroupsGroupFull] = Field(...)
    next_from: Optional[str] = Field(None, description="New from value")


class NewsfeedGetListsExtended(BaseModel):
    count: int = Field(..., description="Total number")
    items: List[objects.NewsfeedListFull] = Field(...)


class NewsfeedGetLists(BaseModel):
    count: int = Field(..., description="Total number")
    items: List[objects.NewsfeedList] = Field(...)


class NewsfeedGetMentions(BaseModel):
    count: int = Field(..., description="Total number")
    items: List[objects.WallWallpostToId] = Field(...)


class NewsfeedGetRecommended(BaseModel):
    items: Optional[List[objects.NewsfeedNewsfeedItem]] = Field(None)
    profiles: Optional[List[objects.UsersUserFull]] = Field(None)
    groups: Optional[List[objects.GroupsGroupFull]] = Field(None)
    new_offset: Optional[str] = Field(None, description="New offset value")
    next_from: Optional[str] = Field(None, description="Next from value")


class NewsfeedGetSuggestedSources(BaseModel):
    count: Optional[int] = Field(None, description="Total number")
    items: Optional[List[Union[objects.GroupsGroupFull, objects.UsersUserXtrType]]] = Field(None)


class NewsfeedGet(BaseModel):
    items: Optional[List[objects.NewsfeedNewsfeedItem]] = Field(None)
    profiles: Optional[List[objects.UsersUserFull]] = Field(None)
    groups: Optional[List[objects.GroupsGroupFull]] = Field(None)
    next_from: Optional[str] = Field(None, description="New from value")


NewsfeedSaveList = int


class NewsfeedSearchExtended(BaseModel):
    items: Optional[List[objects.WallWallpostFull]] = Field(None)
    profiles: Optional[List[objects.UsersUserFull]] = Field(None)
    groups: Optional[List[objects.GroupsGroupFull]] = Field(None)
    suggested_queries: Optional[List[str]] = Field(None)
    next_from: Optional[str] = Field(None)
    count: Optional[int] = Field(None, description="Filtered number")
    total_count: Optional[int] = Field(None, description="Total number")


class NewsfeedSearch(BaseModel):
    items: Optional[List[objects.WallWallpostFull]] = Field(None)
    suggested_queries: Optional[List[str]] = Field(None)
    next_from: Optional[str] = Field(None)
    count: Optional[int] = Field(None, description="Filtered number")
    total_count: Optional[int] = Field(None, description="Total number")


NotesAdd = int


NotesCreateComment = int


class NotesGetById(objects.NotesNote):
    ...


class NotesGetComments(BaseModel):
    count: int = Field(..., description="Total number")
    items: List[objects.NotesNoteComment] = Field(...)


class NotesGet(BaseModel):
    count: int = Field(..., description="Total number")
    items: List[objects.NotesNote] = Field(...)


class NotificationsGet(BaseModel):
    count: Optional[int] = Field(None, description="Total number")
    # items: Optional[List[objects.NotificationsNotificationItem]] = Field(None)
    profiles: Optional[List[objects.UsersUser]] = Field(None)
    groups: Optional[List[objects.GroupsGroup]] = Field(None)
    last_viewed: Optional[int] = Field(None, description="Time when user has been checked notifications last time")
    photos: Optional[List[objects.PhotosPhoto]] = Field(None)
    videos: Optional[List[objects.VideoVideo]] = Field(None)
    apps: Optional[List[objects.AppsApp]] = Field(None)
    next_from: Optional[str] = Field(None)
    ttl: Optional[int] = Field(None)


NotificationsMarkAsViewed = objects.BaseBoolInt


NotificationsSendMessage = List[objects.NotificationsSendMessageItem]


OrdersCancelSubscription = objects.BaseBoolInt


OrdersChangeState = str


class OrdersGetAmount(objects.OrdersAmount):
    ...


OrdersGetById = List[objects.OrdersOrder]


class OrdersGetUserSubscriptionById(objects.OrdersSubscription):
    ...


class OrdersGetUserSubscriptions(BaseModel):
    count: Optional[int] = Field(None, description="Total number")
    items: Optional[List[objects.OrdersSubscription]] = Field(None)


OrdersGet = List[objects.OrdersOrder]


OrdersUpdateSubscription = objects.BaseBoolInt


PagesGetHistory = List[objects.PagesWikipageHistory]


PagesGetTitles = List[objects.PagesWikipage]


class PagesGetVersion(objects.PagesWikipageFull):
    ...


class PagesGet(objects.PagesWikipageFull):
    ...


PagesParseWiki = str


PagesSaveAccess = int


PagesSave = int


PhotosCopy = int


class PhotosCreateAlbum(objects.PhotosPhotoAlbumFull):
    ...


PhotosCreateComment = int


PhotosDeleteComment = objects.BaseBoolInt


PhotosGetAlbumsCount = int


class PhotosGetAlbums(BaseModel):
    count: int = Field(..., description="Total number")
    items: List[objects.PhotosPhotoAlbumFull] = Field(...)


class PhotosGetAllComments(BaseModel):
    count: Optional[int] = Field(None, description="Total number")
    items: Optional[List[objects.PhotosCommentXtrPid]] = Field(None)


class PhotosGetAllExtended(BaseModel):
    count: Optional[int] = Field(None, description="Total number")
    items: Optional[List[objects.PhotosPhotoFullXtrRealOffset]] = Field(None)
    more: Optional[objects.BaseBoolInt] = Field(None, description="Information whether next page is presented")


class PhotosGetAll(BaseModel):
    count: Optional[int] = Field(None, description="Total number")
    items: Optional[List[objects.PhotosPhotoXtrRealOffset]] = Field(None)
    more: Optional[objects.BaseBoolInt] = Field(None, description="Information whether next page is presented")


PhotosGetByIdExtended = List[objects.PhotosPhotoFull]


PhotosGetById = List[objects.PhotosPhoto]


class PhotosGetCommentsExtended(BaseModel):
    count: int = Field(..., description="Total number")
    real_offset: Optional[int] = Field(None, description="Real offset of the comments")
    items: List[objects.WallWallComment] = Field(...)
    profiles: List[objects.UsersUserFull] = Field(...)
    groups: List[objects.GroupsGroupFull] = Field(...)


class PhotosGetComments(BaseModel):
    count: Optional[int] = Field(None, description="Total number")
    real_offset: Optional[int] = Field(None, description="Real offset of the comments")
    items: Optional[List[objects.WallWallComment]] = Field(None)


class PhotosGetMarketUploadServer(objects.BaseUploadServer):
    ...


class PhotosGetMessagesUploadServer(objects.PhotosPhotoUpload):
    ...


class PhotosGetNewTags(BaseModel):
    count: int = Field(..., description="Total number")
    items: List[objects.PhotosPhotoXtrTagInfo] = Field(...)


PhotosGetTags = List[objects.PhotosPhotoTag]


class PhotosGetUploadServer(objects.PhotosPhotoUpload):
    ...


class PhotosGetUserPhotosExtended(BaseModel):
    count: int = Field(..., description="Total number")
    items: List[objects.PhotosPhotoFull] = Field(...)


class PhotosGetUserPhotos(BaseModel):
    count: int = Field(..., description="Total number")
    items: List[objects.PhotosPhoto] = Field(...)


class PhotosGetWallUploadServer(objects.PhotosPhotoUpload):
    ...


class PhotosGetExtended(BaseModel):
    count: int = Field(..., description="Total number")
    items: List[objects.PhotosPhotoFull] = Field(...)


class PhotosGet(BaseModel):
    count: int = Field(..., description="Total number")
    items: List[objects.PhotosPhoto] = Field(...)


PhotosPutTag = int


PhotosRestoreComment = objects.BaseBoolInt


PhotosSaveMarketAlbumPhoto = List[objects.PhotosPhoto]


PhotosSaveMarketPhoto = List[objects.PhotosPhoto]


PhotosSaveMessagesPhoto = List[objects.PhotosPhoto]


PhotosSaveOwnerCoverPhoto = List[objects.BaseImage]


class PhotosSaveOwnerPhoto(BaseModel):
    photo_hash: str = Field(..., description="Photo hash")
    photo_src: str = Field(..., description="Uploaded image url")
    photo_src_big: Optional[str] = Field(None, description="Uploaded image url")
    photo_src_small: Optional[str] = Field(None, description="Uploaded image url")
    saved: Optional[int] = Field(None, description="Returns 1 if profile photo is saved")
    post_id: Optional[int] = Field(None, description="Created post ID")


PhotosSaveWallPhoto = List[objects.PhotosPhoto]


PhotosSave = List[objects.PhotosPhoto]


class PhotosSearch(BaseModel):
    count: Optional[int] = Field(None, description="Total number")
    items: Optional[List[objects.PhotosPhoto]] = Field(None)


PollsAddVote = objects.BaseBoolInt


class PollsCreate(objects.PollsPoll):
    ...


PollsDeleteVote = objects.BaseBoolInt


class PollsGetById(objects.PollsPoll):
    ...


PollsGetVoters = List[objects.PollsVoters]


class PrettyCardsCreate(BaseModel):
    owner_id: int = Field(..., description="Owner ID of created pretty card")
    card_id: str = Field(..., description="Card ID of created pretty card")


class PrettyCardsDelete(BaseModel):
    owner_id: int = Field(..., description="Owner ID of deleted pretty card")
    card_id: str = Field(..., description="Card ID of deleted pretty card")
    error: Optional[str] = Field(None, description="Error reason if error happened")


class PrettyCardsEdit(BaseModel):
    owner_id: int = Field(..., description="Owner ID of edited pretty card")
    card_id: str = Field(..., description="Card ID of edited pretty card")


PrettyCardsGetById = List[objects.PrettycardsPrettycard]


PrettyCardsGetUploadUrl = str


class PrettyCardsGet(BaseModel):
    count: int = Field(..., description="Total number")
    items: List[objects.PrettycardsPrettycard] = Field(...)


class SearchGetHints(BaseModel):
    count: int = Field(...)
    items: List[objects.SearchHint] = Field(...)
    suggested_queries: Optional[List[str]] = Field(None)


class SecureCheckToken(objects.SecureTokenChecked):
    ...


SecureGetAppBalance = int


SecureGetSmshistory = List[objects.SecureSmsNotification]


SecureGetTransactionsHistory = List[objects.SecureTransaction]


SecureGetUserLevel = List[objects.SecureLevel]


SecureGiveEventSticker = List[dict]


SecureSendNotification = List[int]


StatsGetPostReach = List[objects.StatsWallpostStat]


StatsGet = List[objects.StatsPeriod]


class StatusGet(objects.StatusStatus):
    ...


StorageGetKeys = List[str]


StorageGet = str


StorageGetV5110 = List[objects.StorageValue]


StorageGetWithKeys = List[objects.StorageValue]


class StoriesGetBannedExtended(BaseModel):
    count: int = Field(..., description="Stories count")
    items: List[int] = Field(...)
    profiles: List[objects.UsersUserFull] = Field(...)
    groups: List[objects.GroupsGroupFull] = Field(...)


class StoriesGetBanned(BaseModel):
    count: int = Field(..., description="Stories count")
    items: List[int] = Field(...)


class StoriesGetByIdExtended(BaseModel):
    count: int = Field(..., description="Stories count")
    items: List[objects.StoriesStory] = Field(...)
    profiles: List[objects.UsersUserFull] = Field(...)
    groups: List[objects.GroupsGroupFull] = Field(...)


class StoriesGetById(BaseModel):
    count: int = Field(..., description="Stories count")
    items: List[objects.StoriesStory] = Field(...)


class StoriesGetPhotoUploadServer(BaseModel):
    upload_url: str = Field(..., description="Upload URL")
    user_ids: List[int] = Field(..., description="Users ID who can to see story.")


class StoriesGetStats(objects.StoriesStoryStats):
    ...


class StoriesGetVideoUploadServer(BaseModel):
    upload_url: str = Field(..., description="Upload URL")
    user_ids: List[int] = Field(..., description="Users ID who can to see story.")


class StoriesGetViewersExtendedV5115(BaseModel):
    count: int = Field(..., description="Viewers count")
    items: List[objects.StoriesViewersItem] = Field(...)
    hidden_reason: Optional[str] = Field(None)


class StoriesGetViewersExtended(BaseModel):
    count: int = Field(..., description="Viewers count")
    items: List[objects.UsersUserFull] = Field(...)


class StoriesGetV5113(BaseModel):
    count: int = Field(...)
    items: List[objects.StoriesFeedItem] = Field(...)
    profiles: Optional[List[objects.UsersUser]] = Field(None)
    groups: Optional[List[objects.GroupsGroup]] = Field(None)
    need_upload_screen: Optional[bool] = Field(None)


class StoriesGet(BaseModel):
    count: int = Field(..., description="Stories count")
    items: List[List[objects.StoriesStory]] = Field(...)
    promo_data: Optional[objects.StoriesPromoBlock] = Field(None)
    profiles: Optional[List[objects.UsersUser]] = Field(None)
    groups: Optional[List[objects.GroupsGroup]] = Field(None)
    need_upload_screen: Optional[bool] = Field(None)


class StoriesUpload(BaseModel):
    upload_result: Optional[str] = Field(None, description="A string hash that is used in the stories.save method")


class StreamingGetServerUrl(BaseModel):
    endpoint: Optional[str] = Field(None, description="Server host")
    key: Optional[str] = Field(None, description="Access key")


class UsersGetFollowersFields(BaseModel):
    count: int = Field(..., description="Total number of available results")
    items: List[objects.UsersUserFull] = Field(...)


class UsersGetFollowers(BaseModel):
    count: int = Field(..., description="Total friends number")
    items: List[int] = Field(...)


class UsersGetSubscriptionsExtended(BaseModel):
    count: int = Field(..., description="Total number of available results")
    items: List[objects.UsersSubscriptionsItem] = Field(...)


class UsersGetSubscriptions(BaseModel):
    users: objects.UsersUsersArray = Field(...)
    groups: objects.GroupsGroupsArray = Field(...)


UsersGet = List[objects.UsersUserXtrCounters]


class UsersSearch(BaseModel):
    count: Optional[int] = Field(None, description="Total number of available results")
    items: Optional[List[objects.UsersUserFull]] = Field(None)


class UtilsCheckLink(objects.UtilsLinkChecked):
    ...


class UtilsGetLastShortenedLinks(BaseModel):
    count: Optional[int] = Field(None, description="Total number of available results")
    items: Optional[List[objects.UtilsLastShortenedLink]] = Field(None)


class UtilsGetLinkStatsExtended(objects.UtilsLinkStatsExtended):
    ...


class UtilsGetLinkStats(objects.UtilsLinkStats):
    ...


UtilsGetServerTime = int


class UtilsGetShortLink(objects.UtilsShortLink):
    ...


class UtilsResolveScreenName(objects.UtilsDomainResolved):
    ...


class VideoAddAlbum(BaseModel):
    album_id: int = Field(..., description="Created album ID")


VideoCreateComment = int


class VideoGetAlbumById(objects.VideoVideoAlbumFull):
    ...


class VideoGetAlbumsByVideoExtended(BaseModel):
    count: Optional[int] = Field(None, description="Total number")
    items: Optional[List[objects.VideoVideoAlbumFull]] = Field(None)


VideoGetAlbumsByVideo = List[int]


class VideoGetAlbumsExtended(BaseModel):
    count: int = Field(..., description="Total number")
    items: List[objects.VideoVideoAlbumFull] = Field(...)


class VideoGetAlbums(BaseModel):
    count: int = Field(..., description="Total number")
    items: List[objects.VideoVideoAlbumFull] = Field(...)


class VideoGetCommentsExtended(BaseModel):
    count: int = Field(..., description="Total number")
    items: List[objects.WallWallComment] = Field(...)
    profiles: List[objects.UsersUserMin] = Field(...)
    groups: List[objects.GroupsGroupFull] = Field(...)


class VideoGetComments(BaseModel):
    count: int = Field(..., description="Total number")
    items: List[objects.WallWallComment] = Field(...)


class VideoGetExtended(BaseModel):
    count: int = Field(..., description="Total number")
    items: List[objects.VideoVideoFull] = Field(...)
    profiles: List[objects.UsersUserMin] = Field(...)
    groups: List[objects.GroupsGroupFull] = Field(...)


class VideoGet(BaseModel):
    count: int = Field(..., description="Total number")
    items: List[objects.VideoVideo] = Field(...)


VideoRestoreComment = objects.BaseBoolInt


class VideoSave(objects.VideoSaveResult):
    ...


class VideoSearchExtended(BaseModel):
    count: int = Field(..., description="Total number")
    items: List[objects.VideoVideo] = Field(...)
    profiles: List[objects.UsersUserMin] = Field(...)
    groups: List[objects.GroupsGroupFull] = Field(...)


class VideoSearch(BaseModel):
    count: int = Field(..., description="Total number")
    items: List[objects.VideoVideo] = Field(...)


class WallCreateComment(BaseModel):
    comment_id: int = Field(..., description="Created comment ID")


class WallEdit(BaseModel):
    post_id: int = Field(..., description="Edited post ID")


class WallGetByIdExtended(BaseModel):
    items: List[objects.WallWallpostFull] = Field(...)
    profiles: List[objects.UsersUserFull] = Field(...)
    groups: List[objects.GroupsGroupFull] = Field(...)


WallGetById = List[objects.WallWallpostFull]


class WallGetCommentExtended(BaseModel):
    items: List[objects.WallWallComment] = Field(...)
    profiles: List[objects.UsersUser] = Field(...)
    groups: List[objects.GroupsGroup] = Field(...)


class WallGetComment(BaseModel):
    items: List[objects.WallWallComment] = Field(...)


class WallGetCommentsExtended(BaseModel):
    count: int = Field(..., description="Total number")
    items: List[objects.WallWallComment] = Field(...)
    show_reply_button: Optional[bool] = Field(None)
    can_post: Optional[bool] = Field(None, description="Information whether current user can comment the post")
    groups_can_post: Optional[bool] = Field(None, description="Information whether groups can comment the post")
    current_level_count: Optional[int] = Field(None, description="Count of replies of current level")
    profiles: List[objects.UsersUser] = Field(...)
    groups: List[objects.GroupsGroup] = Field(...)


class WallGetComments(BaseModel):
    count: int = Field(..., description="Total number")
    items: List[objects.WallWallComment] = Field(...)
    can_post: Optional[bool] = Field(None, description="Information whether current user can comment the post")
    groups_can_post: Optional[bool] = Field(None, description="Information whether groups can comment the post")
    current_level_count: Optional[int] = Field(None, description="Count of replies of current level")


class WallGetReposts(BaseModel):
    items: List[objects.WallWallpostFull] = Field(...)
    profiles: List[objects.UsersUser] = Field(...)
    groups: List[objects.GroupsGroup] = Field(...)


class WallGetExtended(BaseModel):
    count: int = Field(..., description="Total number")
    items: List[objects.WallWallpostFull] = Field(...)
    profiles: List[objects.UsersUserFull] = Field(...)
    groups: List[objects.GroupsGroupFull] = Field(...)


class WallGet(BaseModel):
    count: int = Field(..., description="Total number")
    items: List[objects.WallWallpostFull] = Field(...)


class WallPostAdsStealth(BaseModel):
    post_id: int = Field(..., description="Created post ID")


class WallPost(BaseModel):
    post_id: int = Field(..., description="Created post ID")


class WallRepost(BaseModel):
    success: int = Field(...)
    post_id: int = Field(..., description="Created post ID")
    reposts_count: int = Field(..., description="Reposts number")
    likes_count: int = Field(..., description="Reposts number")


class WallSearchExtended(BaseModel):
    count: int = Field(..., description="Total number")
    items: List[objects.WallWallpostFull] = Field(...)
    profiles: List[objects.UsersUserFull] = Field(...)
    groups: List[objects.GroupsGroupFull] = Field(...)


class WallSearch(BaseModel):
    count: int = Field(..., description="Total number")
    items: List[objects.WallWallpostFull] = Field(...)


class WidgetsGetComments(BaseModel):
    count: int = Field(..., description="Total number")
    posts: List[objects.WidgetsWidgetComment] = Field(...)


class WidgetsGetPages(BaseModel):
    count: int = Field(..., description="Total number")
    pages: List[objects.WidgetsWidgetPage] = Field(...)



# This file was autogenerated from vk-api json schema

from typing import List, Union, Optional, overload
from easy_vk.types import objects
from easy_vk.types import responses
from easy_vk.api_category import BaseCategory
try:
    from typing import Literal
except Exception:
    from typing_extensions import Literal


class Utils(BaseCategory):
    def __init__(self, session, access_token: str, v: str, last_call_timer, delay: float, auto_retry: bool, max_retries: int, timeout: float):
        super().__init__(session, access_token, v, last_call_timer, delay, auto_retry, max_retries, timeout)

    def check_link(self, url: str) -> responses.UtilsCheckLink:
        """
        Checks whether a link is blocked in VK.
        
        :param url: Link to check (e.g., 'http://google.com').
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'utils.checkLink'
        response_type = responses.UtilsCheckLink
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def delete_from_last_shortened(self, key: str) -> responses.BaseOk:
        """
        Deletes shortened link from user's list.
        
        :param key: Link key (characters after vk.cc/).
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'utils.deleteFromLastShortened'
        response_type = responses.BaseOk
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def get_last_shortened_links(self, count: Optional[int] = None, offset: Optional[int] = None) -> responses.UtilsGetLastShortenedLinks:
        """
        Returns a list of user's shortened links.
        
        :param count: Number of links to return.
        :param offset: Offset needed to return a specific subset of links.
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'utils.getLastShortenedLinks'
        response_type = responses.UtilsGetLastShortenedLinks
        return self._call(method_name, method_parameters, param_aliases, response_type)

    @overload
    def get_link_stats(self, key: str, source: Optional[str] = None, access_key: Optional[str] = None, interval: Optional[str] = None, intervals_count: Optional[int] = None, extended: None = None) -> responses.UtilsGetLinkStats: ...
    @overload
    def get_link_stats(self, key: str, extended: bool, source: Optional[str] = None, access_key: Optional[str] = None, interval: Optional[str] = None, intervals_count: Optional[int] = None) -> responses.UtilsGetLinkStatsExtended: ...
    def get_link_stats(self, key: str, source: Optional[str] = None, access_key: Optional[str] = None, interval: Optional[str] = None, intervals_count: Optional[int] = None, extended: Optional[bool] = None):
        """
        Returns stats data for shortened link.
        
        :param key: Link key (characters after vk.cc/).
        :param source: Source of scope
        :param access_key: Access key for private link stats.
        :param interval: Interval.
        :param intervals_count: Number of intervals to return.
        :param extended: 1 — to return extended stats data (sex, age, geo). 0 — to return views number only.
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'utils.getLinkStats'
        if not extended:
            response_type = responses.UtilsGetLinkStats
        if extended:
            response_type = responses.UtilsGetLinkStatsExtended
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def get_server_time(self) -> responses.UtilsGetServerTime:
        """
        Returns the current time of the VK server.
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'utils.getServerTime'
        response_type = responses.UtilsGetServerTime
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def get_short_link(self, url: str, private: Optional[bool] = None) -> responses.UtilsGetShortLink:
        """
        Allows to receive a link shortened via vk.cc.
        
        :param url: URL to be shortened.
        :param private: 1 — private stats, 0 — public stats.
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'utils.getShortLink'
        response_type = responses.UtilsGetShortLink
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def resolve_screen_name(self, screen_name: str) -> responses.UtilsResolveScreenName:
        """
        Detects a type of object (e.g., user, community, application) and its ID by screen name.
        
        :param screen_name: Screen name of the user, community (e.g., 'apiclub,' 'andrew', or 'rules_of_war'), or application.
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'utils.resolveScreenName'
        response_type = responses.UtilsResolveScreenName
        return self._call(method_name, method_parameters, param_aliases, response_type)



# This file was autogenerated from vk-api json schema

from typing import List, Union, Optional, overload
from easy_vk.types import objects
from easy_vk.types import responses
from easy_vk.api_category import BaseCategory
try:
    from typing import Literal
except Exception:
    from typing_extensions import Literal


class Docs(BaseCategory):
    def __init__(self, session, access_token: str, v: str, last_call_timer, delay: float, auto_retry: bool, max_retries: int, timeout: float):
        super().__init__(session, access_token, v, last_call_timer, delay, auto_retry, max_retries, timeout)

    def get_messages_upload_server(self, type_: Optional[str] = None, peer_id: Optional[int] = None) -> responses.BaseGetUploadServer:
        """
        Returns the server address for document upload.
        
        :param type_: Document type.
        :param peer_id: Destination ID. "For user: 'User ID', e.g. '12345'. For chat: '2000000000' + 'Chat ID', e.g. '2000000001'. For community: '- Community ID', e.g. '-12345'. "
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = [('type_', 'type')]
        method_name = 'docs.getMessagesUploadServer'
        response_type = responses.BaseGetUploadServer
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def get_wall_upload_server(self, group_id: Optional[int] = None) -> responses.BaseGetUploadServer:
        """
        Returns the server address for document upload onto a user's or community's wall.
        
        :param group_id: Community ID (if the document will be uploaded to the community).
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'docs.getWallUploadServer'
        response_type = responses.BaseGetUploadServer
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def save(self, file: str, title: Optional[str] = None, tags: Optional[str] = None, return_tags: Optional[bool] = None) -> responses.DocsSave:
        """
        Saves a document after [vk.com/dev/upload_files_2|uploading it to a server].
        
        :param file: This parameter is returned when the file is [vk.com/dev/upload_files_2|uploaded to the server].
        :param title: Document title.
        :param tags: Document tags.
        :param return_tags:
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'docs.save'
        response_type = responses.DocsSave
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def search(self, q: str, search_own: Optional[bool] = None, count: Optional[int] = None, offset: Optional[int] = None, return_tags: Optional[bool] = None) -> responses.DocsSearch:
        """
        Returns a list of documents matching the search criteria.
        
        :param q: Search query string.
        :param search_own:
        :param count: Number of results to return.
        :param offset: Offset needed to return a specific subset of results.
        :param return_tags:
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'docs.search'
        response_type = responses.DocsSearch
        return self._call(method_name, method_parameters, param_aliases, response_type)



# This file was autogenerated from vk-api json schema

from typing import List, Union, Optional, overload
from easy_vk.types import objects
from easy_vk.types import responses
from easy_vk.api_category import BaseCategory
try:
    from typing import Literal
except Exception:
    from typing_extensions import Literal


class Notes(BaseCategory):
    def __init__(self, session, access_token: str, v: str, last_call_timer, delay: float, auto_retry: bool, max_retries: int, timeout: float):
        super().__init__(session, access_token, v, last_call_timer, delay, auto_retry, max_retries, timeout)

    def add(self, title: str, text: str, privacy_view: Optional[List[str]] = None, privacy_comment: Optional[List[str]] = None) -> responses.NotesAdd:
        """
        Creates a new note for the current user.
        
        :param title: Note title.
        :param text: Note text.
        :param privacy_view:
        :param privacy_comment:
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'notes.add'
        response_type = responses.NotesAdd
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def create_comment(self, note_id: int, message: str, owner_id: Optional[int] = None, reply_to: Optional[int] = None, guid: Optional[str] = None) -> responses.NotesCreateComment:
        """
        Adds a new comment on a note.
        
        :param note_id: Note ID.
        :param message: Comment text.
        :param owner_id: Note owner ID.
        :param reply_to: ID of the user to whom the reply is addressed (if the comment is a reply to another comment).
        :param guid:
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'notes.createComment'
        response_type = responses.NotesCreateComment
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def delete(self, note_id: int) -> responses.BaseOk:
        """
        Deletes a note of the current user.
        
        :param note_id: Note ID.
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'notes.delete'
        response_type = responses.BaseOk
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def delete_comment(self, comment_id: int, owner_id: Optional[int] = None) -> responses.BaseOk:
        """
        Deletes a comment on a note.
        
        :param comment_id: Comment ID.
        :param owner_id: Note owner ID.
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'notes.deleteComment'
        response_type = responses.BaseOk
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def edit(self, note_id: int, title: str, text: str, privacy_view: Optional[List[str]] = None, privacy_comment: Optional[List[str]] = None) -> responses.BaseOk:
        """
        Edits a note of the current user.
        
        :param note_id: Note ID.
        :param title: Note title.
        :param text: Note text.
        :param privacy_view:
        :param privacy_comment:
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'notes.edit'
        response_type = responses.BaseOk
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def edit_comment(self, comment_id: int, message: str, owner_id: Optional[int] = None) -> responses.BaseOk:
        """
        Edits a comment on a note.
        
        :param comment_id: Comment ID.
        :param message: New comment text.
        :param owner_id: Note owner ID.
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'notes.editComment'
        response_type = responses.BaseOk
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def get(self, note_ids: Optional[List[int]] = None, user_id: Optional[int] = None, offset: Optional[int] = None, count: Optional[int] = None, sort: Optional[int] = None) -> responses.NotesGet:
        """
        Returns a list of notes created by a user.
        
        :param note_ids: Note IDs.
        :param user_id: Note owner ID.
        :param offset:
        :param count: Number of notes to return.
        :param sort:
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'notes.get'
        response_type = responses.NotesGet
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def get_by_id(self, note_id: int, owner_id: Optional[int] = None, need_wiki: Optional[bool] = None) -> responses.NotesGetById:
        """
        Returns a note by its ID.
        
        :param note_id: Note ID.
        :param owner_id: Note owner ID.
        :param need_wiki:
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'notes.getById'
        response_type = responses.NotesGetById
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def get_comments(self, note_id: int, owner_id: Optional[int] = None, sort: Optional[int] = None, offset: Optional[int] = None, count: Optional[int] = None) -> responses.NotesGetComments:
        """
        Returns a list of comments on a note.
        
        :param note_id: Note ID.
        :param owner_id: Note owner ID.
        :param sort:
        :param offset:
        :param count: Number of comments to return.
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'notes.getComments'
        response_type = responses.NotesGetComments
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def restore_comment(self, comment_id: int, owner_id: Optional[int] = None) -> responses.BaseOk:
        """
        Restores a deleted comment on a note.
        
        :param comment_id: Comment ID.
        :param owner_id: Note owner ID.
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'notes.restoreComment'
        response_type = responses.BaseOk
        return self._call(method_name, method_parameters, param_aliases, response_type)



# This file was autogenerated from vk-api json schema

from typing import List, Union, Optional, overload
from easy_vk.types import objects
from easy_vk.types import responses
from easy_vk.api_category import BaseCategory
try:
    from typing import Literal
except Exception:
    from typing_extensions import Literal


class Auth(BaseCategory):
    def __init__(self, session, access_token: str, v: str, last_call_timer, delay: float, auto_retry: bool, max_retries: int, timeout: float):
        super().__init__(session, access_token, v, last_call_timer, delay, auto_retry, max_retries, timeout)

    def check_phone(self, phone: str, client_id: Optional[int] = None, client_secret: Optional[str] = None, auth_by_phone: Optional[bool] = None) -> responses.BaseOk:
        """
        Checks a user's phone number for correctness.
        
        :param phone: Phone number.
        :param client_id: User ID.
        :param client_secret:
        :param auth_by_phone:
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'auth.checkPhone'
        response_type = responses.BaseOk
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def restore(self, phone: str, last_name: str) -> responses.AuthRestore:
        """
        Allows to restore account access using a code received via SMS. " This method is only available for apps with [vk.com/dev/auth_direct|Direct authorization] access. "
        
        :param phone: User phone number.
        :param last_name: User last name.
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'auth.restore'
        response_type = responses.AuthRestore
        return self._call(method_name, method_parameters, param_aliases, response_type)



from .streaming import Streaming
from .utils import Utils
from .widgets import Widgets
from .board import Board
from .ads import Ads
from .storage import Storage
from .stats import Stats
from .auth import Auth
from .docs import Docs
from .stories import Stories
from .video import Video
from .wall import Wall
from .pages import Pages
from .database import Database
from .apps import Apps
from .users import Users
from .messages import Messages
from .photos import Photos
from .gifts import Gifts
from .newsfeed import Newsfeed
from .search import Search
from .market import Market
from .account import Account
from .leads import Leads
from .friends import Friends
from .groups import Groups
from .orders import Orders
from .downloadedGames import Downloadedgames
from .notes import Notes
from .prettyCards import Prettycards
from .fave import Fave
from .likes import Likes
from .polls import Polls
from .status import Status
from .notifications import Notifications
